@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2011-2018, Intel Corporation
+* Copyright (c) 2011-2020, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
 #include "codechal_decode_sfc_avc.h"
 #include "codechal_mmc_decode_avc.h"
 #include "codechal_secure_decode_interface.h"
+#include "hal_oca_interface.h"
 #if USE_CODECHAL_DEBUG_TOOL
 #include "codechal_debug.h"
 #endif
@@ -77,8 +78,8 @@ MOS_STATUS CodechalDecodeAvc::SendSlice(
         if (!m_mfxInterface->IsAvcISlice(slc->slice_type))
         {
             refIdxParams.CurrPic = avcPicParams->CurrPic;
-            refIdxParams.uiList = 0;
-            refIdxParams.uiNumRefForList = slc->num_ref_idx_l0_active_minus1 + 1;
+            refIdxParams.uiList = LIST_0;
+            refIdxParams.uiNumRefForList[LIST_0] = slc->num_ref_idx_l0_active_minus1 + 1;
 
             CODECHAL_DECODE_CHK_STATUS_MESSAGE_RETURN(MOS_SecureMemcpy(
                 &refIdxParams.RefPicList,
@@ -112,8 +113,8 @@ MOS_STATUS CodechalDecodeAvc::SendSlice(
 
             if (m_mfxInterface->IsAvcBSlice(slc->slice_type))
             {
-                refIdxParams.uiList = 1;
-                refIdxParams.uiNumRefForList = slc->num_ref_idx_l1_active_minus1 + 1;
+                refIdxParams.uiList = LIST_1;
+                refIdxParams.uiNumRefForList[LIST_1] = slc->num_ref_idx_l1_active_minus1 + 1;
                 CODECHAL_DECODE_CHK_STATUS_RETURN(m_mfxInterface->AddMfxAvcRefIdx(cmdBuffer, nullptr, &refIdxParams));
 
                 if (avcPicParams->pic_fields.weighted_bipred_idc == 1)
@@ -136,6 +137,7 @@ MOS_STATUS CodechalDecodeAvc::SendSlice(
         {
             MHW_VDBOX_AVC_REF_IDX_PARAMS refIdxParams;
             MOS_ZeroMemory(&refIdxParams, sizeof(MHW_VDBOX_AVC_REF_IDX_PARAMS));
+            refIdxParams.bDummyReference = true;
             CODECHAL_DECODE_CHK_STATUS_RETURN(m_mfxInterface->AddMfxAvcRefIdx(cmdBuffer, nullptr, &refIdxParams));
         }
 
@@ -149,7 +151,7 @@ MOS_STATUS CodechalDecodeAvc::SendSlice(
     return eStatus;
 }
 
-MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture()
+MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture(PMOS_SURFACE surface)
 {
     MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
 
@@ -162,7 +164,15 @@ MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture()
     MOS_SURFACE dstSurface;
     MOS_ZeroMemory(&dstSurface, sizeof(MOS_SURFACE));
     dstSurface.Format = Format_NV12;
-    dstSurface.OsResource = m_decodeParams.m_destSurface->OsResource;
+    if(surface != nullptr && !Mos_ResourceIsNull(&surface->OsResource))
+    {
+       dstSurface.OsResource = surface->OsResource;
+    }
+    else
+    {
+       CODECHAL_DECODE_ASSERTMESSAGE("Surface pointer is NULL!");
+       return MOS_STATUS_INVALID_PARAMETER;
+    }
     CODECHAL_DECODE_CHK_STATUS_RETURN(CodecHalGetResourceInfo(m_osInterface, &dstSurface));
 
     uint32_t height = dstSurface.dwHeight;
@@ -200,7 +210,7 @@ MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture()
 
         MOS_ZeroMemory(&hucStreamOutParams, sizeof(hucStreamOutParams));
         hucStreamOutParams.dataBuffer            = &m_resMonoPictureChromaBuffer;
-        hucStreamOutParams.streamOutObjectBuffer = &m_decodeParams.m_destSurface->OsResource;
+        hucStreamOutParams.streamOutObjectBuffer = &surface->OsResource;
     }
 
     uint32_t uvblockHeight = CODECHAL_MACROBLOCK_HEIGHT;
@@ -218,7 +228,7 @@ MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture()
             dataCopyParams.srcResource = &m_resMonoPictureChromaBuffer;
             dataCopyParams.srcSize     = uvrowSize;
             dataCopyParams.srcOffset   = 0;
-            dataCopyParams.dstResource = &m_decodeParams.m_destSurface->OsResource;
+            dataCopyParams.dstResource = &surface->OsResource;
             dataCopyParams.dstSize     = frameSize;
             dataCopyParams.dstOffset   = dstOffset;
 
@@ -229,7 +239,7 @@ MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture()
             CODECHAL_DECODE_CHK_STATUS_RETURN(HucCopy(
                 &cmdBuffer,                                 // pCmdBuffer
                 &m_resMonoPictureChromaBuffer,              // presSrc
-                &m_decodeParams.m_destSurface->OsResource,  // presDst
+                &surface->OsResource,                      // presDst
                 uvrowSize,                                  // u32CopyLength
                 0,                                          // u32CopyInputOffset
                 dstOffset));                                // u32CopyOutputOffset
@@ -246,7 +256,7 @@ MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture()
         dataCopyParams.srcResource     = &m_resMonoPictureChromaBuffer;
         dataCopyParams.srcSize         = uvsize;
         dataCopyParams.srcOffset       = 0;
-        dataCopyParams.dstResource     = &m_decodeParams.m_destSurface->OsResource;
+        dataCopyParams.dstResource     = &surface->OsResource;
         dataCopyParams.dstSize         = frameSize;
         dataCopyParams.dstOffset       = dstOffset;
 
@@ -257,7 +267,7 @@ MOS_STATUS CodechalDecodeAvc::FormatAvcMonoPicture()
         CODECHAL_DECODE_CHK_STATUS_RETURN(HucCopy(
             &cmdBuffer,                                 // pCmdBuffer
             &m_resMonoPictureChromaBuffer,              // presSrc
-            &m_decodeParams.m_destSurface->OsResource,  // presDst
+            &surface->OsResource,                      // presDst
             uvsize,                                     // u32CopyLength
             0,                                          // u32CopyInputOffset
             dstOffset));                                // u32CopyOutputOffset
@@ -398,15 +408,30 @@ MOS_STATUS CodechalDecodeAvc::InitMvcDummyDmvBuffer(
     uint32_t i, numMBs = size / 64;
     for (i = 0; i<numMBs; i++)
     {
-        CODECHAL_DECODE_CHK_STATUS_RETURN(MOS_SecureMemcpy(mbDmvBuffer, 64, mvcWaDummyDmvBuf, 64));
+        eStatus = (MOS_STATUS)MOS_SecureMemcpy(mbDmvBuffer, 64, mvcWaDummyDmvBuf, 64);
+        if (eStatus != MOS_STATUS_SUCCESS)
+        {
+            MOS_SafeFreeMemory(dummyDmvBuffer);
+            CODECHAL_DECODE_CHK_STATUS_RETURN(eStatus);
+        }
         mbDmvBuffer += 64;
     }
 
     CodechalResLock ResourceLock(m_osInterface, mvcDummyDmvBuffer);
     auto data = (uint8_t*)ResourceLock.Lock(CodechalResLock::writeOnly);
-    CODECHAL_DECODE_CHK_NULL_RETURN(data);
 
-    CODECHAL_DECODE_CHK_STATUS_RETURN(MOS_SecureMemcpy(data, size, (void*)dummyDmvBuffer, size));
+    if (data  == nullptr)
+    {
+        MOS_FreeMemory(dummyDmvBuffer);
+        CODECHAL_DECODE_CHK_NULL_RETURN(nullptr);
+    }
+
+    eStatus = (MOS_STATUS)MOS_SecureMemcpy(data, size, (void*)dummyDmvBuffer, size);
+    if (eStatus != MOS_STATUS_SUCCESS)
+    {
+        MOS_SafeFreeMemory(dummyDmvBuffer);
+        CODECHAL_DECODE_CHK_STATUS_RETURN(eStatus);
+    }
 
     MOS_FreeMemAndSetNull(dummyDmvBuffer);
     return eStatus;
@@ -954,9 +979,9 @@ MOS_STATUS CodechalDecodeAvc::AllocateResourcesFixedSizes()
     MOS_ZeroMemory(&lockFlagsWriteOnly, sizeof(MOS_LOCK_PARAMS));
     lockFlagsWriteOnly.WriteOnly = 1;
 
-    CodecHalAllocateDataList(
+    CODECHAL_DECODE_CHK_STATUS_RETURN(CodecHalAllocateDataList(
         m_avcRefList,
-        CODEC_AVC_NUM_UNCOMPRESSED_SURFACE);
+        CODEC_AVC_NUM_UNCOMPRESSED_SURFACE));
 
     m_currPic.PicFlags = PICTURE_INVALID;
     m_currPic.FrameIdx = CODEC_AVC_NUM_UNCOMPRESSED_SURFACE;
@@ -1201,7 +1226,7 @@ MOS_STATUS CodechalDecodeAvc::SetFrameStates()
 
     CODECHAL_DECODE_CHK_STATUS_RETURN(SetPictureStructs());
 
-    CODECHAL_DECODE_CHK_STATUS_RETURN(FormatAvcMonoPicture());
+    CODECHAL_DECODE_CHK_STATUS_RETURN(FormatAvcMonoPicture(m_decodeParams.m_destSurface));
 
     if (m_avcPicParams->pic_fields.IntraPicFlag)
     {
@@ -1218,7 +1243,10 @@ MOS_STATUS CodechalDecodeAvc::SetFrameStates()
     auto decProcessingParams = (CODECHAL_DECODE_PROCESSING_PARAMS *)m_decodeParams.m_procParams;
     if (decProcessingParams != nullptr)
     {
-        CODECHAL_DECODE_CHK_NULL_RETURN(m_fieldScalingInterface);
+        if (!decProcessingParams->bIsReferenceOnlyPattern && m_downsamplingHinted)
+        {
+            CODECHAL_DECODE_CHK_NULL_RETURN(m_fieldScalingInterface);
+        }
 
         CODECHAL_DECODE_CHK_STATUS_RETURN(m_sfcState->CheckAndInitialize(
             decProcessingParams,
@@ -1228,19 +1256,48 @@ MOS_STATUS CodechalDecodeAvc::SetFrameStates()
             m_deblockingEnabled));
 
         if (!((!CodecHal_PictureIsFrame(m_avcPicParams->CurrPic) ||
-                  m_avcPicParams->seq_fields.mb_adaptive_frame_field_flag) &&
-                m_fieldScalingInterface->IsFieldScalingSupported(decProcessingParams)) &&
-            m_sfcState->m_sfcPipeOut == false)
+             m_avcPicParams->seq_fields.mb_adaptive_frame_field_flag) &&
+             (m_downsamplingHinted && m_fieldScalingInterface->IsFieldScalingSupported(decProcessingParams))) &&
+             m_sfcState->m_sfcPipeOut == false &&
+            !decProcessingParams->bIsReferenceOnlyPattern)
         {
-            eStatus = MOS_STATUS_UNKNOWN;
-            CODECHAL_DECODE_ASSERTMESSAGE("Downsampling parameters are NOT supported!");
-            return eStatus;
+            m_vdSfcSupported = false;
+        }
+        else
+        {
+            m_vdSfcSupported = true;
         }
     }
 #endif
     m_crrPic = m_avcPicParams->CurrPic;
-    m_secondField =
-        CodecHal_PictureIsBottomField(m_avcPicParams->CurrPic);
+
+    if(m_fullFieldsFrame == (PICTURE_BOTTOM_FIELD | PICTURE_TOP_FIELD))
+    {
+        m_fullFieldsFrame = 0;
+    }
+
+    m_secondField = false;
+
+    if(CodecHal_PictureIsField(m_avcPicParams->CurrPic))
+    {
+       if(CodecHal_PictureIsTopField(m_avcPicParams->CurrPic))
+       {
+           m_fullFieldsFrame |= PICTURE_TOP_FIELD;
+       }
+       if (CodecHal_PictureIsBottomField(m_avcPicParams->CurrPic))
+       {
+           m_fullFieldsFrame |= PICTURE_BOTTOM_FIELD;
+       }
+    }
+    else
+    {
+        m_fullFieldsFrame = 0;
+    }
+
+    if(m_fullFieldsFrame == (PICTURE_BOTTOM_FIELD | PICTURE_TOP_FIELD))
+    {
+        m_secondField = true;
+    }
 
     CODECHAL_DEBUG_TOOL(
         m_debugInterface->m_currPic     = m_crrPic;
@@ -1327,7 +1384,9 @@ MOS_STATUS CodechalDecodeAvc::InitPicMhwParams(
         picMhwParams->PipeBufAddrParams.psPreDeblockSurface = &m_destSurface;
     }
 
+#ifdef _MMC_SUPPORTED
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->SetPipeBufAddr(&picMhwParams->PipeBufAddrParams));
+#endif
 
     picMhwParams->PipeBufAddrParams.presMfdIntraRowStoreScratchBuffer =
         &m_resMfdIntraRowStoreScratchBuffer;
@@ -1402,9 +1461,11 @@ MOS_STATUS CodechalDecodeAvc::InitPicMhwParams(
         }
     }
 
+#ifdef _MMC_SUPPORTED
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->CheckReferenceList(&picMhwParams->PipeBufAddrParams));
 
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->SetRefrenceSync(m_disableDecodeSyncLock, m_disableLockForTranscode));
+#endif
 
     CODECHAL_DECODE_CHK_STATUS_RETURN(MOS_SecureMemcpy(picMhwParams->PipeBufAddrParams.presReferences, sizeof(PMOS_RESOURCE) * CODEC_AVC_MAX_NUM_REF_FRAME, m_presReferences, sizeof(PMOS_RESOURCE) * CODEC_AVC_MAX_NUM_REF_FRAME));
 
@@ -1513,13 +1574,15 @@ MOS_STATUS CodechalDecodeAvc::DecodeStateLevel()
     PIC_MHW_PARAMS picMhwParams;
     CODECHAL_DECODE_CHK_STATUS_RETURN(InitPicMhwParams(&picMhwParams));
 
+    auto mmioRegisters = m_hwInterface->GetMfxInterface()->GetMmioRegisters(m_vdboxIndex);
+    HalOcaInterface::On1stLevelBBStart(cmdBuffer, *m_osInterface->pOsContext, m_osInterface->CurrentGpuContextHandle, *m_miInterface, *mmioRegisters);
+
     if (m_cencBuf && m_cencBuf->checkStatusRequired)
     {
         CODECHAL_DECODE_COND_ASSERTMESSAGE((m_vdboxIndex > m_hwInterface->GetMfxInterface()->GetMaxVdboxIndex()), "ERROR - vdbox index exceed the maximum");
-        auto mmioRegisters = m_hwInterface->GetMfxInterface()->GetMmioRegisters(m_vdboxIndex);
 
         CODECHAL_DECODE_CHK_STATUS_RETURN(m_hwInterface->GetCpInterface()->CheckStatusReportNum(
-            mmioRegisters, 
+            mmioRegisters,
             m_cencBuf->bufIdx,
             m_cencBuf->resStatus,
             &cmdBuffer));
@@ -1798,7 +1861,7 @@ MOS_STATUS CodechalDecodeAvc::DecodePrimitiveLevel()
                 }
             }
 
-            decodeStatusReport.m_secondField = CodecHal_PictureIsBottomField(m_avcPicParams->CurrPic);
+            decodeStatusReport.m_secondField = m_secondField;
             decodeStatusReport.m_frameType   = m_perfType;)
 
         CODECHAL_DECODE_CHK_STATUS_RETURN(EndStatusReport(decodeStatusReport, &cmdBuffer));
@@ -1834,6 +1897,7 @@ MOS_STATUS CodechalDecodeAvc::DecodePrimitiveLevel()
     //    m_debugInterface,
     //    &cmdBuffer));
     )
+    HalOcaInterface::On1stLevelBBEnd(cmdBuffer, *m_osInterface);
 
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_osInterface->pfnSubmitCommandBuffer(m_osInterface, &cmdBuffer, m_videoContextUsesNullHw));
 
@@ -1928,11 +1992,9 @@ MOS_STATUS CodechalDecodeAvc::CalcDownsamplingParams(
     *format = Format_NV12;
     *frameIdx = avcPicParams->CurrPic.FrameIdx;
 
-    if (m_refSurfaces == nullptr)
-    {
-        *refSurfWidth = (avcPicParams->pic_width_in_mbs_minus1 + 1) * CODECHAL_MACROBLOCK_WIDTH;
-        *refSurfHeight = (avcPicParams->pic_height_in_mbs_minus1 + 1) * CODECHAL_MACROBLOCK_HEIGHT;
-    }
+    *refSurfWidth = (avcPicParams->pic_width_in_mbs_minus1 + 1) * CODECHAL_MACROBLOCK_WIDTH;
+    *refSurfHeight = (avcPicParams->pic_height_in_mbs_minus1 + 1) * CODECHAL_MACROBLOCK_HEIGHT;
+
 
     return eStatus;
 }
@@ -1972,6 +2034,9 @@ CodechalDecodeAvc::CodechalDecodeAvc(
     MOS_ZeroMemory(&m_resAvcDmvBuffers, (sizeof(MOS_RESOURCE) * CODEC_AVC_NUM_DMV_BUFFERS));
     MOS_ZeroMemory(&m_resInvalidRefBuffer, sizeof(MOS_RESOURCE));
     MOS_ZeroMemory(&m_resMvcDummyDmvBuffer, (sizeof(MOS_RESOURCE) * 2));
+    MOS_ZeroMemory(&m_destSurface, sizeof(MOS_SURFACE));
+    MOS_ZeroMemory(&m_resSyncObjectWaContextInUse, sizeof(MOS_RESOURCE));
+    MOS_ZeroMemory(&m_resSyncObjectVideoContextInUse, sizeof(MOS_RESOURCE));
     m_refFrameSurface = nullptr;
 
     m_vldSliceRecord = nullptr;
