@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2011-2018, Intel Corporation
+* Copyright (c) 2011-2019, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -121,14 +121,6 @@ typedef enum _CODECHAL_CS_ENGINE_ID_DEF
     CODECHAL_CLASS_ID_VIDEO_ENGINE = 1,
 } CODECHAL_CS_ENGINE_ID_DEF;
 
-typedef enum _CODECHAL_DUMMY_REFERENCE_STATUS
-{
-    CODECHAL_DUMMY_REFERENCE_INVALID,
-    CODECHAL_DUMMY_REFERENCE_DPB,
-    CODECHAL_DUMMY_REFERENCE_DEST_SURFACE,
-    CODECHAL_DUMMY_REFERENCE_ALLOCATED
-} CODECHAL_DUMMY_REFERENCE_STATUS;
-
 typedef union _CODECHAL_CS_ENGINE_ID
 {
     struct
@@ -396,15 +388,15 @@ public:
     //!           Surface name
     //! \param    [in] format
     //!           Surface format, by default is NV12
-    //! \param    [in] isCompressed
-    //!           Compress flag, by default is false
+    //! \param    [in] isCompressible
+    //!           Compressible flag, by default is false
     MOS_STATUS AllocateSurface(
         PMOS_SURFACE surface,
         uint32_t width,
         uint32_t height,
         const char* name,
         MOS_FORMAT format = Format_NV12,
-        bool isCompressed = false);
+        bool isCompressible = false);
 
     //!
     //! \brief  Entry to allocate and intialize the decode instance
@@ -415,13 +407,6 @@ public:
     //!
     MOS_STATUS Allocate(CodechalSetting * codecHalSettings) override;
 
-    //!
-    //! \brief  The handle at the begining of each frame.
-    //! \return MOS_STATUS
-    //!         MOS_STATUS_SUCCESS if success, else fail reason
-    //!
-    MOS_STATUS BeginFrame() override;
-
     //!
     //! \brief  The handle at the end of each frame.
     //! \return MOS_STATUS
@@ -582,9 +567,9 @@ public:
 
     //!
     //! \brief  Indicates whether or not the first execute call
-    //! \return If first execute call \see m_firstExecuteCall
+    //! \return If first execute call \see m_executeCallIndex
     //!
-    bool IsFirstExecuteCall() { return m_firstExecuteCall; }
+    inline bool IsFirstExecuteCall() { return (m_executeCallIndex == 0); }
 
     //!
     //! \brief  Indicates whether or not the decoder is inuse
@@ -622,18 +607,6 @@ public:
     //!
     bool GetRenderContextUsesNullHw() { return m_renderContextUsesNullHw; }
 
-    //!
-    //! \brief  Get Huc product family
-    //! \return Huc prodoct family
-    //!
-    uint32_t GetHuCProductFamily() { return m_huCProductFamily; }
-
-    //!
-    //! \brief  Set Huc product family
-    //! \return N/A
-    //!
-    void SetHuCProductFamily(uint32_t huCProductFamily) { m_huCProductFamily = huCProductFamily; }
-
     //!
     //! \brief  Set decode histogram
     //! \return No return
@@ -647,6 +620,18 @@ public:
     CodechalDecodeHistogram* GetDecodeHistogram() { return m_decodeHistogram; }
 
 #ifdef _DECODE_PROCESSING_SUPPORTED
+    //!
+    //! \brief  Indicates whether or not the vd sfc is supported
+    //! \return If vd sfc is supported \see m_vdSfcSupported
+    //!
+    bool IsVdSfcSupported() { return m_vdSfcSupported; }
+
+    //!
+    //! \brief  Set if vd sfc supported
+    //! \return No return
+    //!
+    void SetVdSfcSupportedFlag(bool isVdSfcSpported) { m_vdSfcSupported = isVdSfcSpported; }
+
     //! \brief Field scaling interface
     FieldScalingInterface       *m_fieldScalingInterface = nullptr;
 #endif
@@ -672,6 +657,12 @@ public:
         m_dummyReferenceStatus = status;
     }
 
+    //!
+    //! \brief  Get mmc enable flag
+    //! \return bool
+    //!
+    bool IsDecoderMmcEnabled(){return m_mmc ? m_mmc->IsMmcEnabled() : false;}
+
 protected:
 
     //!
@@ -901,7 +892,33 @@ private:
         uint32_t allocWidth,
         uint32_t allocHeight,
         MOS_FORMAT format);
-
+    //!
+    //! \brief    Resize specific reference surfaces
+    //! \details  Resize specific reference surfaces for decode downsampling for all codec types
+    //! \param    frameIdx
+    //!           [in] index of surfaces array
+    //! \param    width
+    //!           [in] Width of the surfaces to be allocated
+    //! \param    height
+    //!           [in] Height of the surfaces to be allocated
+    //! \param    format
+    //!           [in] Flag to indicate the format of the surfaces to be allocated
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if success, else fail reason
+    //!
+   MOS_STATUS RefSurfacesResize(
+        uint32_t     frameIdx,
+        uint32_t     width,
+        uint32_t     height,
+        MOS_FORMAT   format);
+    //!
+    //! \brief    Deallocate specific reference surfaces
+    //! \details  Deallocate specific reference surfaces for decode downsampling for all codec types
+    //! \param    frameIdx
+    //!           [in] index of surfaces array
+    //! \return   N/A
+    //!
+    void DeallocateSpecificRefSurfaces(uint32_t frameIdx);
     //!
     //! \brief    Deallocate reference surfaces
     //! \details  Deallocate reference surfaces for decode downsampling for all codec types
@@ -949,8 +966,10 @@ protected:
     uint32_t                    m_standard          = CODECHAL_UNDEFINED;
     //! \brief Current frame number
     uint32_t                    m_frameNum          = 0;
-    //! \brief Indicates if current field is second field(bottom field)
+    //! \brief Indicates if current field is second field
     bool                        m_secondField       = false;
+    //! \brief Indidates if fields are completed
+    uint32_t                    m_fullFieldsFrame     = 0;
     //! \brief picture information of current render target
     CODEC_PICTURE               m_crrPic            = {0};
     //! \brief Video GPU node inuse
@@ -976,8 +995,8 @@ protected:
     uint32_t                    m_standardDecodePatchListSizeNeeded = 0;
     //! \brief Indicates if current input bitstream is incomplete
     bool                        m_incompletePicture = false;
-    //! \brief Indicates if current is frist execution call in multiple execution call mode
-    bool                        m_firstExecuteCall  = false;
+    //! \brief The index of execution call in multiple execution call mode
+    bool                        m_executeCallIndex  = 0;
     //! \brief Indicates if current is frist execution call in multiple execution call mode
     bool                        m_consecutiveMbErrorConcealmentInUse = false;
     //! \brief Indicates if phantom MBs is required for MPEG2 decode
@@ -988,6 +1007,11 @@ protected:
     //! \brief Flag to indicate if UMD Perf Profiler FE BE timing measurement is enabled
     bool                        m_perfFEBETimingEnabled = false;
 
+#ifdef _DECODE_PROCESSING_SUPPORTED
+    //! \brief Flag to indicate if vd sfc is supported
+    bool                        m_vdSfcSupported = false;
+#endif
+
     //! \brief Stores all the status_query related data
     CodechalDecodeStatusBuffer  m_decodeStatusBuf;
     //! \brief The feedback number reported by app in picparams call
@@ -1020,9 +1044,9 @@ protected:
     bool                        m_disableLockForTranscode = false;
 
     //! \brief Indicate how many passes is needed to finish decoding a picture
-    //! \details Initialize decode pass number to 1, for those decoder need more than 1 decoding pass, 
+    //! \details Initialize decode pass number to 1, for those decoder need more than 1 decoding pass,
     //!          modify this value in specific decoder files.
-    uint8_t                     m_decodePassNum     = 1;
+    uint16_t                     m_decodePassNum     = 1;
 
     //! \brief MMIO Hcp Frame CRC report offset
     uint32_t                    m_hcpFrameCrcRegOffset = 0;
@@ -1050,9 +1074,6 @@ protected:
     PMOS_SURFACE                m_downsampledSurfaces = nullptr;
 #endif
 
-    //! \brief Huc product family
-    uint32_t                    m_huCProductFamily = HUC_UNKNOWN;
-
     //! \brief    Decode histogram interface
     //! \details  Support YUV Luma histogram.
     CodechalDecodeHistogram    *m_decodeHistogram = nullptr;
