@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2009-2017, Intel Corporation
+* Copyright (c) 2009-2019, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -32,17 +32,27 @@
 #include "mos_os.h"
 #include "mos_util_debug.h"
 #include "mos_util_user_interface.h"
+#include "mos_interface.h"
 
-//!
-//! \brief GLOBAL INITIALIZERS
-//!
-const MOS_USER_FEATURE g_MosUserFeatureInit =
+PerfUtility* g_perfutility = PerfUtility::getInstance();
+
+AutoPerfUtility::AutoPerfUtility(std::string tag, std::string comp, std::string level)
 {
-    MOS_USER_FEATURE_TYPE_INVALID,                                              // Type
-    nullptr,                                                                       // pSettingPath
-    nullptr,                                                                       // pSettingsValues
-    0                                                                           // uiNumSettingValues
-};
+    if (PERFUTILITY_IS_ENABLED(comp, level))
+    {
+        g_perfutility->startTick(tag);
+        autotag = tag;
+        bEnable = true;
+    }
+}
+
+AutoPerfUtility::~AutoPerfUtility()
+{
+    if (bEnable)
+    {
+        g_perfutility->stopTick(autotag);
+    }
+}
 
 #if MOS_MEDIASOLO_SUPPORTED
 void *   _MOS_INTERFACE::pvSoloContext = nullptr; 
@@ -65,36 +75,7 @@ MOS_STATUS Mos_AddCommand(
     const void              *pCmd,
     uint32_t                dwCmdSize)
 {
-    uint32_t dwCmdSizeDwAligned = 0;
-
-    //---------------------------------------------
-    MOS_OS_CHK_NULL_RETURN(pCmdBuffer);
-    MOS_OS_CHK_NULL_RETURN(pCmd);
-    //---------------------------------------------
-
-    if (dwCmdSize == 0)
-    {
-        MOS_OS_ASSERTMESSAGE("Incorrect command size to add to command buffer.");
-        return MOS_STATUS_INVALID_PARAMETER;
-    }
-
-    dwCmdSizeDwAligned = MOS_ALIGN_CEIL(dwCmdSize, sizeof(uint32_t));
-
-    pCmdBuffer->iOffset    += dwCmdSizeDwAligned;
-    pCmdBuffer->iRemaining -= dwCmdSizeDwAligned;
-
-    if (pCmdBuffer->iRemaining < 0)
-    {
-        pCmdBuffer->iOffset    -= dwCmdSizeDwAligned;
-        pCmdBuffer->iRemaining += dwCmdSizeDwAligned;
-        MOS_OS_ASSERTMESSAGE("Unable to add command (no space).");
-        return MOS_STATUS_UNKNOWN;
-    }
-
-    MOS_SecureMemcpy(pCmdBuffer->pCmdPtr, dwCmdSize, pCmd, dwCmdSize);
-    pCmdBuffer->pCmdPtr += (dwCmdSizeDwAligned / sizeof(uint32_t));
-
-    return MOS_STATUS_SUCCESS;
+    return MosInterface::AddCommand(pCmdBuffer, pCmd, dwCmdSize);
 }
 
 //!
@@ -377,10 +358,17 @@ MOS_STATUS Mos_DumpCommandBuffer(
     char            sFileName[MOS_MAX_HLT_FILENAME_LEN];
     // Maximum length of engine name is 6
     char            sEngName[6];
+    char            *psFileNameAfterPrefix  = nullptr;
+    size_t          nSizeFileNamePrefix     = 0;
 
     MOS_OS_CHK_NULL_RETURN(pOsInterface);
     MOS_OS_CHK_NULL_RETURN(pCmdBuffer);
 
+    if (pOsInterface->apoMosEnabled)
+    {
+        return MosInterface::DumpCommandBuffer(pOsInterface->osStreamState, pCmdBuffer);
+    }
+
     // Set the name of the engine that is going to be used.
     MOS_GPU_CONTEXT sGpuContext = pOsInterface->pfnGetGpuContext(pOsInterface);
     switch (sGpuContext)
@@ -392,13 +380,22 @@ MOS_STATUS Mos_DumpCommandBuffer(
         case MOS_GPU_CONTEXT_VDBOX2_VIDEO:
         case MOS_GPU_CONTEXT_VDBOX2_VIDEO2:
         case MOS_GPU_CONTEXT_VDBOX2_VIDEO3:
+        case MOS_GPU_CONTEXT_VIDEO5:
+        case MOS_GPU_CONTEXT_VIDEO6:
+        case MOS_GPU_CONTEXT_VIDEO7:
             MOS_SecureStrcpy(sEngName, sizeof(sEngName), MOS_COMMAND_BUFFER_VIDEO_ENGINE);
             break;
+        case MOS_GPU_CONTEXT_TEE:
+            MOS_SecureStrcpy(sEngName, sizeof(sEngName), MOS_COMMAND_BUFFER_TEE_ENGINE);
+            break;
         case MOS_GPU_CONTEXT_RENDER:
         case MOS_GPU_CONTEXT_RENDER2:
         case MOS_GPU_CONTEXT_RENDER3:
         case MOS_GPU_CONTEXT_RENDER4:
+        case MOS_GPU_CONTEXT_RENDER_RA:
         case MOS_GPU_CONTEXT_COMPUTE:
+        case MOS_GPU_CONTEXT_COMPUTE_RA:
+        case MOS_GPU_CONTEXT_CM_COMPUTE:
             MOS_SecureStrcpy(sEngName, sizeof(sEngName), MOS_COMMAND_BUFFER_RENDER_ENGINE);
             break;
         case MOS_GPU_CONTEXT_VEBOX:
@@ -429,19 +426,14 @@ MOS_STATUS Mos_DumpCommandBuffer(
     if (pOsInterface->bDumpCommandBufferToFile)
     {
         // Set the file name.
-        eStatus = MOS_LogFileNamePrefix(sFileName);
-        if (eStatus != MOS_STATUS_SUCCESS)
-        {
-            MOS_OS_NORMALMESSAGE("Failed to create log file prefix. Status = %d", eStatus);
-            goto finish;
-        }
-
+        memcpy(sFileName, pOsInterface->sDirName, MOS_MAX_HLT_FILENAME_LEN);
+        nSizeFileNamePrefix = strnlen(sFileName, sizeof(sFileName));
         MOS_SecureStringPrint(
-            sFileName,
-            sizeof(sFileName),
-            sizeof(sFileName),
-            "%s%c%s%c%s_%d.txt",
-            sFileName, MOS_DIR_SEPERATOR, MOS_COMMAND_BUFFER_OUT_DIR,
+            sFileName + nSizeFileNamePrefix,
+            sizeof(sFileName) - nSizeFileNamePrefix,
+            sizeof(sFileName) - nSizeFileNamePrefix,
+            "%c%s%c%s_%d.txt",
+            MOS_DIR_SEPERATOR, MOS_COMMAND_BUFFER_OUT_DIR,
             MOS_DIR_SEPERATOR, MOS_COMMAND_BUFFER_OUT_FILE, dwCommandBufferNumber);
 
         // Write the output buffer to file.
@@ -519,6 +511,8 @@ MOS_STATUS Mos_DumpCommandBufferInit(
     char                                sFileName[MOS_MAX_HLT_FILENAME_LEN];
     MOS_STATUS                          eStatus = MOS_STATUS_UNKNOWN;
     MOS_USER_FEATURE_VALUE_DATA         UserFeatureData;
+    char                                *psFileNameAfterPrefix = nullptr;
+    size_t                              nSizeFileNamePrefix = 0;
 
     MOS_OS_CHK_NULL_RETURN(pOsInterface);
 
@@ -536,19 +530,21 @@ MOS_STATUS Mos_DumpCommandBufferInit(
     if (pOsInterface->bDumpCommandBufferToFile)
     {
         // Create output directory.
-        eStatus = MOS_LogFileNamePrefix(sFileName);
+        eStatus = MOS_LogFileNamePrefix(pOsInterface->sDirName);
         if (eStatus != MOS_STATUS_SUCCESS)
         {
             MOS_OS_NORMALMESSAGE("Failed to create log file prefix. Status = %d", eStatus);
             goto finish;
         }
 
+        memcpy(sFileName, pOsInterface->sDirName, MOS_MAX_HLT_FILENAME_LEN);
+        nSizeFileNamePrefix = strnlen(sFileName, sizeof(sFileName));
         MOS_SecureStringPrint(
-            sFileName,
-            sizeof(sFileName),
-            sizeof(sFileName),
-            "%s%c%s",
-            sFileName, MOS_DIR_SEPERATOR, MOS_COMMAND_BUFFER_OUT_DIR);
+            sFileName + nSizeFileNamePrefix,
+            sizeof(sFileName) - nSizeFileNamePrefix,
+            sizeof(sFileName) - nSizeFileNamePrefix,
+            "%c%s",
+            MOS_DIR_SEPERATOR, MOS_COMMAND_BUFFER_OUT_DIR);
 
         eStatus = MOS_CreateDirectory(sFileName);
         if (eStatus != MOS_STATUS_SUCCESS)
@@ -713,9 +709,6 @@ MOS_STATUS Mos_InitInterface(
     pOsUserFeatureInterface = &pOsInterface->UserFeatureInterface;
     MOS_OS_CHK_NULL_RETURN(pOsUserFeatureInterface);
 
-    // Setup Member variables
-    pOsUserFeatureInterface->pUserFeatureInit = &g_MosUserFeatureInit;
-
     // Setup Member functions
     pOsInterface->pfnFillResource       = Mos_OsFillResource;
     pOsInterface->pfnWaitOnResource     = Mos_OsWaitOnResource;
@@ -726,9 +719,14 @@ MOS_STATUS Mos_InitInterface(
     pOsInterface->veDefaultEnable       = true;
 
     pOsInterface->streamIndex = 0;
+    pOsInterface->bSimIsActive = 0;
 
     eStatus = Mos_Specific_InitInterface(pOsInterface, pOsDriverContext);
-
+    if (eStatus != MOS_STATUS_SUCCESS)
+    {
+        MOS_OS_ASSERTMESSAGE("Mos_Specific_InitInterface FAILED, errno = 0x%x", eStatus);
+        return eStatus;
+    }
 #if MOS_COMMAND_BUFFER_DUMP_SUPPORTED
     Mos_DumpCommandBufferInit(pOsInterface);
 #endif // MOS_COMMAND_BUFFER_DUMP_SUPPORTED
@@ -741,6 +739,38 @@ MOS_STATUS Mos_InitInterface(
         &UserFeatureWriteData,
         1);
 
+    // Apo wrapper
+    if (pOsInterface->apoMosEnabled && !pOsInterface->streamStateIniter)
+    {
+        pOsInterface->osStreamState->component                = pOsInterface->Component;
+        pOsInterface->osStreamState->currentGpuContextHandle  = pOsInterface->CurrentGpuContextHandle;
+        pOsInterface->osStreamState->mediaReset               = pOsInterface->bMediaReset;
+        pOsInterface->osStreamState->nullHwAccelerationEnable = pOsInterface->NullHWAccelerationEnable;
+        pOsInterface->osStreamState->osDeviceContext          = (OsDeviceContext *)pOsInterface->pOsContext->m_osDeviceContext;
+        pOsInterface->osStreamState->simIsActive              = pOsInterface->bSimIsActive;
+        pOsInterface->osStreamState->virtualEngineInterface   = nullptr; // Will be updated by HAL on demand
+#if MOS_COMMAND_BUFFER_DUMP_SUPPORTED
+        pOsInterface->osStreamState->dumpCommandBuffer        = pOsInterface->bDumpCommandBuffer;
+        pOsInterface->osStreamState->dumpCommandBufferAsMessages = pOsInterface->bDumpCommandBufferAsMessages;
+        pOsInterface->osStreamState->dumpCommandBufferToFile  = pOsInterface->bDumpCommandBufferToFile;
+#endif  // MOS_COMMAND_BUFFER_DUMP_SUPPORTED
+
+#if _DEBUG || _RELEASE_INTERNAL
+        pOsInterface->osStreamState->enableDbgOvrdInVirtualEngine = pOsInterface->bEnableDbgOvrdInVE;
+        pOsInterface->osStreamState->eForceVdbox = pOsInterface->eForceVdbox;
+        pOsInterface->osStreamState->eForceVebox = pOsInterface->eForceVebox;
+#endif  // _DEBUG || _RELEASE_INTERNAL
+
+        pOsInterface->osStreamState->ctxBasedScheduling       = pOsInterface->ctxBasedScheduling;
+        pOsInterface->osStreamState->multiNodeScaling         = pOsInterface->multiNodeScaling;
+        pOsInterface->osStreamState->perStreamParameters      = pOsInterface->pOsContext;
+    }
+
+    if (pOsInterface->apoMosEnabled)
+    {
+        pOsInterface->osStreamState->osCpInterface = pOsInterface->osCpInterface;
+    }
+
     return eStatus;
 }
 
@@ -777,6 +807,7 @@ MEMORY_OBJECT_CONTROL_STATE Mos_CachePolicyGetMemoryObject(
         GMM_RESOURCE_USAGE_SURFACE_REF_ENCODE,
         GMM_RESOURCE_USAGE_SURFACE_MV_DATA_ENCODE,
         GMM_RESOURCE_USAGE_SURFACE_HME_DOWNSAMPLED_ENCODE,
+        GMM_RESOURCE_USAGE_SURFACE_HME_DOWNSAMPLED_ENCODE_FF,
         GMM_RESOURCE_USAGE_SURFACE_HME_DOWNSAMPLED_ENCODE_DST,
         GMM_RESOURCE_USAGE_SURFACE_ME_DISTORTION_ENCODE,
         GMM_RESOURCE_USAGE_SURFACE_BRC_ME_DISTORTION_ENCODE,
@@ -855,12 +886,15 @@ MEMORY_OBJECT_CONTROL_STATE Mos_CachePolicyGetMemoryObject(
         GMM_RESOURCE_USAGE_SIZE_STREAMOUT_CODEC,
         GMM_RESOURCE_USAGE_COMPRESSED_HEADER_BUFFER_CODEC,
         GMM_RESOURCE_USAGE_PROBABILITY_DELTA_BUFFER_CODEC,
+        GMM_RESOURCE_USAGE_TILE_RECORD_BUFFER_CODEC,
+        GMM_RESOURCE_USAGE_TILE_SIZE_STAS_BUFFER_CODEC,
         GMM_RESOURCE_USAGE_END_CODEC,
 
         //
         // CM USAGES
         //
         CM_RESOURCE_USAGE_SurfaceState,
+        CM_RESOURCE_USAGE_StateHeap,
         CM_RESOURCE_USAGE_NO_L3_SurfaceState,
         CM_RESOURCE_USAGE_NO_LLC_ELLC_SurfaceState,
         CM_RESOURCE_USAGE_NO_LLC_SurfaceState,
@@ -875,7 +909,11 @@ MEMORY_OBJECT_CONTROL_STATE Mos_CachePolicyGetMemoryObject(
         //
         MP_RESOURCE_USAGE_BEGIN,
         MP_RESOURCE_USAGE_DEFAULT,
+        MP_RESOURCE_USAGE_DEFAULT_FF,
+        MP_RESOURCE_USAGE_DEFAULT_RCS,
         MP_RESOURCE_USAGE_SurfaceState,
+        MP_RESOURCE_USAGE_SurfaceState_FF,
+        MP_RESOURCE_USAGE_SurfaceState_RCS,
         MP_RESOURCE_USAGE_AGE3_SurfaceState,
         MP_RESOURCE_USAGE_EDRAM_SurfaceState,
         MP_RESOURCE_USAGE_EDRAM_AGE3_SurfaceState,
@@ -892,6 +930,9 @@ MEMORY_OBJECT_CONTROL_STATE Mos_CachePolicyGetMemoryObject(
         MHW_RESOURCE_USAGE_Sfc_AvsLineBufferSurface,                                //!< SFC AVS Line buffer Surface
         MHW_RESOURCE_USAGE_Sfc_IefLineBufferSurface,                                //!< SFC IEF Line buffer Surface
 
+        // Camera capture
+        GMM_RESOURCE_USAGE_CAMERA_CAPTURE,
+
     };
 
     MOS_OS_ASSERT(pGmmClientContext);
@@ -918,7 +959,7 @@ MOS_STATUS Mos_CheckVirtualEngineSupported(
     PLATFORM                    platform;
     MOS_USER_FEATURE_VALUE_DATA userFeatureData;
 
-    MOS_OS_ASSERT(osInterface);
+    MOS_OS_CHK_NULL_RETURN(osInterface);
     MOS_ZeroMemory(&platform, sizeof(PLATFORM));
 
     osInterface->pfnGetPlatform(osInterface, &platform);
@@ -937,7 +978,7 @@ MOS_STATUS Mos_CheckVirtualEngineSupported(
 
         // force bSupportVirtualEngine to false when virtual engine not enabled by default
         if ((!veDefaultEnable || !osInterface->veDefaultEnable) && 
-            (eStatus == MOS_STATUS_USER_FEATURE_KEY_READ_FAILED || eStatus == MOS_STATUS_USER_FEATURE_KEY_OPEN_FAILED))
+            (eStatus == MOS_STATUS_USER_FEATURE_KEY_OPEN_FAILED))
         {
             osInterface->bSupportVirtualEngine = false;
         }
@@ -952,6 +993,14 @@ MOS_STATUS Mos_CheckVirtualEngineSupported(
         {
             osInterface->ctxBasedScheduling = false;
         }
+
+        if(osInterface->pfnCheckVirtualEngineSupported)
+        {
+            osInterface->pfnCheckVirtualEngineSupported(osInterface);
+        }
+
+        osInterface->multiNodeScaling = osInterface->ctxBasedScheduling && MEDIA_IS_SKU(skuTable, FtrVcs2) ? true : false;
+
 #if (_DEBUG || _RELEASE_INTERNAL)
         MOS_USER_FEATURE_VALUE_WRITE_DATA  userFeatureWriteData = __NULL_USER_FEATURE_VALUE_WRITE_DATA__;
         userFeatureWriteData.Value.i32Data = osInterface->ctxBasedScheduling ? true : false;
@@ -985,9 +1034,18 @@ MOS_STATUS Mos_CheckVirtualEngineSupported(
         {
             osInterface->ctxBasedScheduling = false;
         }
+        osInterface->multiNodeScaling = osInterface->ctxBasedScheduling && MEDIA_IS_SKU(skuTable, FtrVcs2) ? true : false;
     }
 
+    if (osInterface->apoMosEnabled)
+    {
+        // Update ctx based scheduling flag also in APO MOS stream state
+        MOS_OS_CHK_NULL_RETURN(osInterface->osStreamState);
+        osInterface->osStreamState->ctxBasedScheduling = osInterface->ctxBasedScheduling;
+        osInterface->osStreamState->multiNodeScaling   = osInterface->multiNodeScaling;
+    }
     MOS_OS_VERBOSEMESSAGE("Virtual Engine Context based SCheduling enabled:%d.\n", osInterface->ctxBasedScheduling);
+    MOS_OS_VERBOSEMESSAGE("Virtual Engine Multi-node Scaling enabled:%d.\n", osInterface->multiNodeScaling);
 
     return eStatus;
 }
