@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2009-2019, Intel Corporation
+* Copyright (c) 2009-2020, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -888,19 +888,31 @@ typedef struct _VPHAL_HVSDENOISE_PARAMS
     uint32_t            dwDenoiseParamSize;
 } VPHAL_HVSDENOISE_PARAMS, *PVPHAL_HVSDENOISE_PARAMS;
 
+//!
+//! Structure VPHAL_SLIMIPU_DENOISE_PARAM
+//! \brief SLIMIPU Denoise Parameters
+//!
+typedef struct _VPHAL_SLIMIPU_DENOISE_PARAM
+{
+    uint32_t            MemSizeInBytes;
+    void *              pSystemMem;
+} VPHAL_SLIMIPU_DENOISE_PARAM, *PVPHAL_SLIMIPU_DENOISE_PARAM;
+
 //!
 //! Structure VPHAL_DENOISE_PARAMS
 //! \brief Denoise parameters
 //!
 typedef struct _VPHAL_DENOISE_PARAMS
 {
-    bool                            bEnableChroma;
-    bool                            bEnableLuma;
-    bool                            bAutoDetect;
-    float                           fDenoiseFactor;
-    VPHAL_NOISELEVEL                NoiseLevel;
-    bool                            bEnableHVSDenoise;
-    VPHAL_HVSDENOISE_PARAMS         HVSDenoise;
+    bool                                bEnableChroma;
+    bool                                bEnableLuma;
+    bool                                bAutoDetect;
+    float                               fDenoiseFactor;
+    VPHAL_NOISELEVEL                    NoiseLevel;
+    bool                                bEnableHVSDenoise;
+    VPHAL_HVSDENOISE_PARAMS             HVSDenoise;
+    bool                                bEnableSlimIPUDenoise;
+    VPHAL_SLIMIPU_DENOISE_PARAM         SlimIPUDenoise;
 } VPHAL_DENOISE_PARAMS, *PVPHAL_DENOISE_PARAMS;
 
 //!
@@ -969,6 +981,7 @@ struct VPHAL_SURFACE
     bool                        bFastColorFill = false;           //!< enable fast color fill without copy surface
     bool                        bMaxRectChanged = false;          //!< indicate rcMaxSrc been updated
     bool                        b16UsrPtr = false;                //!< is 16 byte aligned system linear memory.
+    bool                        bVEBOXCroppingUsed = false;       //!< Vebox crop case need use rcSrc as vebox input.
 
     // Interlaced Scaling
     bool                        bInterlacedScaling = false;            //!< Interlaced scaling
@@ -997,6 +1010,8 @@ struct VPHAL_SURFACE
     uint32_t                    dwHeight = 0;                                 //!<  Surface height
     uint32_t                    dwPitch = 0;                                  //!<  Surface pitch
     MOS_TILE_TYPE               TileType = MOS_TILE_X;                        //!<  Tile Type
+    MOS_TILE_MODE_GMM           TileModeGMM = MOS_TILE_LINEAR_GMM;            //!<  Tile Mode from GMM Definition
+    bool                        bGMMTileEnabled = false;                      //!<  GMM Tile Mode Flag
     bool                        bOverlay= false;                              //!<  Overlay Surface
     bool                        bFlipChain = false;                           //!<  FlipChain Surface
     VPHAL_PLANE_OFFSET          YPlaneOffset =  { 0, 0, 0, 0 };               //!<  Y surface plane offset
@@ -1185,7 +1200,8 @@ struct VPHAL_RENDER_PARAMS
     bool                                    bPathKernel;                // HDR path config if use kernel
     bool                                    bAPGWorkloadEnable = false;         //!< Identify Whether APG workload Enabled or not
 
-    bool                                    bDisableVpFor8K = false;
+    bool                                    bDisableVeboxFor8K = false;
+    bool                                    bUseVEHdrSfc       = false;  // use SFC for to perform CSC/Scaling/RGBSwap of HDR streaming; if false, use composite render.
 
     VPHAL_RENDER_PARAMS() :
         uSrcCount(0),
@@ -1208,7 +1224,8 @@ struct VPHAL_RENDER_PARAMS
 #endif
         bCalculatingAlpha(false),
         pExtensionData(nullptr),
-        bPathKernel(false)
+        bPathKernel(false),
+        bUseVEHdrSfc(false)
     {
     }
 
