@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2015-2018, Intel Corporation
+* Copyright (c) 2015-2020, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -576,7 +576,7 @@ MOS_STATUS VpHal_RndrCommonSubmitCommands(
         }
     }
 
-    HalOcaInterface::On1stLevelBBEnd(CmdBuffer, *pOsContext);
+    HalOcaInterface::On1stLevelBBEnd(CmdBuffer, *pOsInterface);
 
     if (pBatchBuffer)
     {
@@ -850,7 +850,7 @@ MOS_STATUS VpHal_RndrSubmitCommands(
         }
     }
 
-    HalOcaInterface::On1stLevelBBEnd(CmdBuffer, *pOsContext);
+    HalOcaInterface::On1stLevelBBEnd(CmdBuffer, *pOsInterface);
 
     if (pBatchBuffer)
     {
