@@ -19,7 +19,7 @@
 //!
 #include "codechal_decoder.h"
 #include "codechal_decode_scalability_g12.h"
-#include "mos_util_user_interface_g12.h"
+#include "media_user_settings_mgr_g12.h"
 #include "mos_os_virtualengine_next.h"
 
 //==<Functions>=======================================================
@@ -450,7 +450,7 @@ MOS_STATUS CodecHalDecodeScalability_InitScalableParams_G12(
     {
         if (!MOS_VE_CTXBASEDSCHEDULING_SUPPORTED(pOsInterface))
         {
-            if (g_apoMosEnabled)
+            if (pOsInterface->apoMosEnabled)
             {
                 CODECHAL_DECODE_CHK_NULL(pVEInterface->veInterface);
                 pScalabilityState->ucScalablePipeNum = pVEInterface->veInterface->GetEngineCount() - 1;
@@ -492,7 +492,8 @@ MOS_STATUS CodecHalDecodeScalability_InitScalableParams_G12(
             u8MaxTileColumn = pScalabilityState->ucScalablePipeNum;
 #endif
         bCanEnableRealTile = bCanEnableRealTile && pInitParams->bIsTileEnabled && (pInitParams->u8NumTileColumns > 1) &&
-            (pInitParams->u8NumTileColumns <= u8MaxTileColumn) && (pInitParams->u8NumTileRows <= HEVC_NUM_MAX_TILE_ROW);
+            (pInitParams->u8NumTileColumns <= u8MaxTileColumn) && (pInitParams->u8NumTileRows <= HEVC_NUM_MAX_TILE_ROW) &&
+            pInitParams->bHasSubsetParams;
         if (bCanEnableRealTile)
         {
             pScalabilityState->bIsRtMode = true;
@@ -514,6 +515,7 @@ MOS_STATUS CodecHalDecodeScalability_InitScalableParams_G12(
         else
         {
             *pucDecPassNum = pScalabilityState->ucScalablePipeNum + 1; // FE + all BEs
+            pScalabilityState->bFESeparateSubmission = pScalabilityState->bFESeparateSubmissionVT;
         }
     }
     else
@@ -650,6 +652,7 @@ MOS_STATUS CodecHalDecodeScalability_FEBESync_G12(
     PMOS_INTERFACE                      pOsInterface;
     MhwMiInterface                      *pMiInterface;
     uint32_t                            HcpDecPhase;
+    uint32_t                            curPipeInUse = 0;
     MOS_STATUS                          eStatus = MOS_STATUS_SUCCESS;
 
     CODECHAL_DECODE_FUNCTION_ENTER;
@@ -711,6 +714,24 @@ MOS_STATUS CodecHalDecodeScalability_FEBESync_G12(
             MHW_MI_SAD_EQUAL_SDD,
             pCmdBufferInUse));
 
+        //add additional BEs Sync for two pipes
+        if (u8PipeNum == 2 && !phasedSubmission)
+        {
+            if (pScalabilityState->bIsRtMode)
+            {
+                curPipeInUse = pScalabilityState->u8RtCurPipe;
+            }
+            else
+            {
+                curPipeInUse = HcpDecPhase == CODECHAL_HCP_DECODE_PHASE_BE0 ? 0 : 1;
+            }
+            //pipe 0: resSemaMemBEsAdditional[0] + 1, pipe 1: resSemaMemBEsAdditional[1] + 1
+            CODECHAL_DECODE_CHK_STATUS(pScalabilityState->pHwInterface->SendMiAtomicDwordCmd(&pScalabilityState->resSemaMemBEsAdditional[curPipeInUse],
+                1,
+                MHW_MI_ATOMIC_INC,
+                pCmdBufferInUse));
+        }
+
         // Program some placeholder cmds to resolve the hazard between BEs sync
         MHW_MI_STORE_DATA_PARAMS dataParams;
         dataParams.pOsResource = &pScalabilityState->resDelayMinus;
@@ -722,9 +743,26 @@ MOS_STATUS CodecHalDecodeScalability_FEBESync_G12(
                 pCmdBufferInUse,
                 &dataParams));
         }
-
+        //additional Semaphore Wait: pipe 0 wait resSemaMemBEsAdditional[1] == 1, pipe 1 wait resSemaMemBEsAdditional[0] == 1
+        if (u8PipeNum == 2 && !phasedSubmission)
+        {
+            CODECHAL_DECODE_CHK_STATUS(pScalabilityState->pHwInterface->SendHwSemaphoreWaitCmd(
+                &pScalabilityState->resSemaMemBEsAdditional[CODECHAL_SCALABILITY_MAX_PIPE_INDEX_OF_TWO_PIPE - curPipeInUse],
+                1,
+                MHW_MI_SAD_EQUAL_SDD,
+                pCmdBufferInUse));
+        }
         //reset HW semaphore
         CODECHAL_DECODE_CHK_STATUS(pScalabilityState->pHwInterface->SendMiAtomicDwordCmd(&pScalabilityState->resSemaMemBEs, 1, MHW_MI_ATOMIC_DEC, pCmdBufferInUse));
+        //pipe 0 reset resSemaMemBEsAdditional[1], pipe 1 reset resSemaMemBEsAdditional[0]
+        if (u8PipeNum == 2 && !phasedSubmission)
+        {
+            CODECHAL_DECODE_CHK_STATUS(pScalabilityState->pHwInterface->SendMiAtomicDwordCmd(
+                &pScalabilityState->resSemaMemBEsAdditional[CODECHAL_SCALABILITY_MAX_PIPE_INDEX_OF_TWO_PIPE - curPipeInUse],
+                1,
+                MHW_MI_ATOMIC_DEC,
+                pCmdBufferInUse));
+        }
 
         if (!pScalabilityState->bIsRtMode)
         {
@@ -840,7 +878,8 @@ MOS_STATUS CodecHalDecodeScalability_InitializeState_G12(
     CodechalDecode                     *pDecoder,
     PCODECHAL_DECODE_SCALABILITY_STATE  pScalabilityStateBase,
     CodechalHwInterface                *hwInterface,
-    bool                                bShortFormat)
+    bool                                bShortFormat,
+    CodechalSetting                    *settings)
 {
     PMOS_VIRTUALENGINE_INTERFACE   pVEInterface;
     MOS_VIRTUALENGINE_INIT_PARAMS  VEInitParms;
@@ -879,6 +918,8 @@ MOS_STATUS CodecHalDecodeScalability_InitializeState_G12(
     pScalabilityState->VideoContextForMP = MOS_VE_MULTINODESCALING_SUPPORTED(osInterface) ? MOS_GPU_CONTEXT_VIDEO5 : MOS_GPU_CONTEXT_VDBOX2_VIDEO;
     pScalabilityState->VideoContextFor3P = MOS_VE_MULTINODESCALING_SUPPORTED(osInterface) ? MOS_GPU_CONTEXT_VIDEO7 : MOS_GPU_CONTEXT_VDBOX2_VIDEO2;
 
+    pScalabilityState->numDelay = 15;
+
 #if (_DEBUG || _RELEASE_INTERNAL)
     // Reg key of the threshold for mode switch single pipe <-> 2 pipe. Using pic width value to control mode switch for now
     MOS_ZeroMemory(&UserFeatureData, sizeof(UserFeatureData));
@@ -953,17 +994,23 @@ MOS_STATUS CodecHalDecodeScalability_InitializeState_G12(
     }
 #endif
 
+    pScalabilityState->bFESeparateSubmissionVT = pScalabilityState->bFESeparateSubmission;
+
     if (pScalabilityState->bFESeparateSubmission)
     {
         MOS_GPU_CONTEXT         GpuContext = MOS_VE_CTXBASEDSCHEDULING_SUPPORTED(osInterface) ? MOS_GPU_CONTEXT_VIDEO : MOS_GPU_CONTEXT_VIDEO4;
         GpuContext                         = MOS_VE_MULTINODESCALING_SUPPORTED(osInterface) ? MOS_GPU_CONTEXT_VIDEO4 : GpuContext;
-        MOS_GPUCTX_CREATOPTIONS createOpts;
 
+        MOS_GPUCTX_CREATOPTIONS_ENHANCED createOpts;
+        createOpts.UsingSFC = settings->sfcInUseHinted && settings->downsamplingHinted
+                              && (MEDIA_IS_SKU(hwInterface->GetSkuTable(), FtrSFCPipe)
+                              && !MEDIA_IS_SKU(hwInterface->GetSkuTable(), FtrDisableVDBox2SFC));
         CODECHAL_DECODE_CHK_STATUS(osInterface->pfnCreateGpuContext(
-            osInterface,
-            GpuContext,
-            MOS_GPU_NODE_VIDEO,
-            &createOpts));
+              osInterface,
+              GpuContext,
+              MOS_GPU_NODE_VIDEO,
+              &createOpts));
+
         pScalabilityState->VideoContextForFE = GpuContext;
     }
 
@@ -1030,7 +1077,7 @@ MOS_STATUS CodecHalDecodeScalability_InitializeState_G12(
 #endif
 
     pScalabilityState->sliceStateCLs = CODECHAL_SCALABILITY_SLICE_STATE_CACHELINES_PER_SLICE_TGL;
-    CODECHAL_DECODE_CHK_STATUS(CodecHalDecodeScalability_AllocateResources_FixedSizes(pScalabilityState));
+    CODECHAL_DECODE_CHK_STATUS(CodecHalDecodeScalability_AllocateResources_FixedSizes_G12(pScalabilityState));
 
 finish:
     return eStatus;
@@ -1395,6 +1442,94 @@ MOS_STATUS CodecHalDecodeScalability_AllocateResources_VariableSizes_G12(
     return eStatus;
 }
 
+MOS_STATUS CodecHalDecodeScalability_AllocateResources_FixedSizes_G12(
+    PCODECHAL_DECODE_SCALABILITY_STATE_G12 pScalabilityState)
+{
+    PMOS_INTERFACE          pOsInterface;
+    MOS_ALLOC_GFXRES_PARAMS AllocParamsForBufferLinear;
+    MOS_LOCK_PARAMS         LockFlagsWriteOnly;
+    uint8_t *               pData = nullptr;
+    MOS_STATUS              eStatus = MOS_STATUS_SUCCESS;
+
+    CODECHAL_DECODE_FUNCTION_ENTER;
+
+    CODECHAL_DECODE_CHK_NULL_RETURN(pScalabilityState);
+    CODECHAL_DECODE_CHK_NULL_RETURN(pScalabilityState->pHwInterface);
+    CODECHAL_DECODE_CHK_NULL_RETURN(pScalabilityState->pHwInterface->GetOsInterface());
+
+    pOsInterface = pScalabilityState->pHwInterface->GetOsInterface();
+
+    MOS_ZeroMemory(&LockFlagsWriteOnly, sizeof(MOS_LOCK_PARAMS));
+    LockFlagsWriteOnly.WriteOnly = 1;
+
+    // initiate allocation paramters
+    MOS_ZeroMemory(&AllocParamsForBufferLinear, sizeof(MOS_ALLOC_GFXRES_PARAMS));
+    AllocParamsForBufferLinear.Type     = MOS_GFXRES_BUFFER;
+    AllocParamsForBufferLinear.TileType = MOS_TILE_LINEAR;
+    AllocParamsForBufferLinear.Format   = Format_Buffer;
+
+    CODECHAL_DECODE_CHK_STATUS_RETURN(CodecHalDecodeScalability_AllocateResources_FixedSizes(pScalabilityState));
+
+    //Allocate additional BEs Semaphore memory for two pipes to avoid race condition
+    if (!pOsInterface->phasedSubmission)
+    {
+        AllocParamsForBufferLinear.pBufName = "AdditionalBESemaphoreMemory";
+        AllocParamsForBufferLinear.dwBytes  = sizeof(uint32_t);
+        for (uint32_t i = 0; i < 2; i++)
+        {
+            eStatus = (MOS_STATUS)pOsInterface->pfnAllocateResource(
+                pOsInterface,
+                &AllocParamsForBufferLinear,
+                &pScalabilityState->resSemaMemBEsAdditional[i]);
+
+            if (eStatus != MOS_STATUS_SUCCESS)
+            {
+                CODECHAL_DECODE_ASSERTMESSAGE("Failed to allocate Additional BE Semaphore memory.");
+                return eStatus;
+            }
+
+            pData = (uint8_t *)pOsInterface->pfnLockResource(
+                pOsInterface,
+                &pScalabilityState->resSemaMemBEsAdditional[i],
+                &LockFlagsWriteOnly);
+
+            CODECHAL_DECODE_CHK_NULL_RETURN(pData);
+
+            MOS_ZeroMemory(pData, sizeof(uint32_t));
+
+            CODECHAL_DECODE_CHK_STATUS_RETURN(pOsInterface->pfnUnlockResource(
+                pOsInterface,
+                &pScalabilityState->resSemaMemBEsAdditional[i]));
+        }
+    }
+
+    return eStatus;
+}
+
+void CodecHalDecodeScalability_Destroy_G12(
+    PCODECHAL_DECODE_SCALABILITY_STATE_G12 pScalabilityState)
+{
+    PMOS_INTERFACE pOsInterface;
+
+    CODECHAL_DECODE_FUNCTION_ENTER;
+
+    CODECHAL_DECODE_CHK_NULL_NO_STATUS_RETURN(pScalabilityState);
+    CODECHAL_DECODE_CHK_NULL_NO_STATUS_RETURN(pScalabilityState->pHwInterface);
+    CODECHAL_DECODE_CHK_NULL_NO_STATUS_RETURN(pScalabilityState->pHwInterface->GetOsInterface());
+    pOsInterface = pScalabilityState->pHwInterface->GetOsInterface();
+
+    CodecHalDecodeScalability_Destroy(pScalabilityState);
+
+    for (int i = 0; i < 2; i++)
+    {
+        pOsInterface->pfnFreeResource(
+            pOsInterface,
+            &pScalabilityState->resSemaMemBEsAdditional[i]);
+    }
+
+    return;
+}
+
 MOS_STATUS CodecHalDecodeScalability_DecidePipeNum_G12(
     PCODECHAL_DECODE_SCALABILITY_STATE         pScalState,
     PCODECHAL_DECODE_SCALABILITY_INIT_PARAMS   pInitParams)
@@ -1420,7 +1555,8 @@ MOS_STATUS CodecHalDecodeScalability_DecidePipeNum_G12(
         u8MaxTileColumn = 2;
 #endif
     bCanEnableRealTile = bCanEnableRealTile && pInitParamsG12->bIsTileEnabled && (pInitParams->u8NumTileColumns > 1) &&
-                         (pInitParams->u8NumTileColumns <= u8MaxTileColumn) && (pInitParams->u8NumTileRows <= HEVC_NUM_MAX_TILE_ROW);
+                         (pInitParams->u8NumTileColumns <= u8MaxTileColumn) && (pInitParams->u8NumTileRows <= HEVC_NUM_MAX_TILE_ROW) &&
+                         pInitParamsG12->bHasSubsetParams;
 
     if (pInitParams->usingSFC)
     {
@@ -1482,33 +1618,7 @@ MOS_STATUS CodecHalDecodeScalability_DecidePipeNum_G12(
             }
             else
             {
-                if (pScalState->dwHcpDecModeSwtichTh1Width != 0 &&
-                    pScalState->dwHcpDecModeSwtichTh2Width != 0)
-                {
-                    if (pInitParams->u32PicWidthInPixel >= pScalState->dwHcpDecModeSwtichTh2Width)
-                    {
-                        pScalState->ucScalablePipeNum = CODECHAL_DECODE_HCP_SCALABLE_PIPE_NUM_RESERVED;
-                    }
-                    else if (pInitParams->u32PicWidthInPixel >= pScalState->dwHcpDecModeSwtichTh1Width)
-                    {
-                        pScalState->ucScalablePipeNum = CODECHAL_DECODE_HCP_SCALABLE_PIPE_NUM_2;
-                    }
-                }
-                else
-                {
-                    if ((pInitParams->u32PicWidthInPixel * pInitParams->u32PicHeightInPixel) >= (CODECHAL_HCP_DECODE_SCALABLE_THRESHOLD4_WIDTH * CODECHAL_HCP_DECODE_SCALABLE_THRESHOLD4_HEIGHT))
-                    {
-                        pScalState->ucScalablePipeNum = CODECHAL_DECODE_HCP_SCALABLE_PIPE_NUM_RESERVED;
-                    }
-                    else if ((!CodechalDecodeNonRextFormat(pInitParams->format)
-                                    && CodechalDecodeResolutionEqualLargerThan4k(pInitParams->u32PicWidthInPixel, pInitParams->u32PicHeightInPixel))
-                                || (CodechalDecodeNonRextFormat(pInitParams->format)
-                                    && CodechalDecodeResolutionEqualLargerThan5k(pInitParams->u32PicWidthInPixel, pInitParams->u32PicHeightInPixel))
-                                || (bCanEnableRealTile && !pInitParams->usingSecureDecode))
-                    {
-                        pScalState->ucScalablePipeNum = CODECHAL_DECODE_HCP_SCALABLE_PIPE_NUM_2;
-                    }
-                }
+                pScalState->ucScalablePipeNum = CODECHAL_DECODE_HCP_Legacy_PIPE_NUM_1;
             }
         }
     }
@@ -1566,14 +1676,16 @@ MOS_STATUS CodechalDecodeScalability_DebugOvrdDecidePipeNum_G12(
     PCODECHAL_DECODE_SCALABILITY_STATE         pScalState)
 {
     PMOS_VIRTUALENGINE_INTERFACE pVEInterface;
-    MOS_STATUS                   eStatus = MOS_STATUS_SUCCESS;
-
+    MOS_STATUS                   eStatus        = MOS_STATUS_SUCCESS;
+    PMOS_INTERFACE               pOsInterface   = nullptr;
     CODECHAL_DECODE_CHK_NULL_RETURN(pScalState);
     CODECHAL_DECODE_CHK_NULL_RETURN(pScalState->pVEInterface);
 
     pVEInterface = pScalState->pVEInterface;
+    pOsInterface = pVEInterface->pOsInterface;
+    CODECHAL_DECODE_CHK_NULL_RETURN(pOsInterface);
 
-    if (g_apoMosEnabled)
+    if (pOsInterface->apoMosEnabled)
     {
         CODECHAL_DECODE_CHK_NULL_RETURN(pVEInterface->veInterface);
         auto veInterface = pVEInterface->veInterface;
