@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2009-2018, Intel Corporation
+* Copyright (c) 2009-2020, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -38,12 +38,15 @@
 #include "media_libva_util.h"
 #include "mos_utilities.h"
 #include "mos_os.h"
+#include "mos_defs.h"
 #include "hwinfo_linux.h"
 #include "media_ddi_decode_base.h"
 #include "media_ddi_encode_base.h"
 #include "media_libva_decoder.h"
 #include "media_libva_encoder.h"
 #include "media_libva_caps.h"
+#include "memory_policy_manager.h"
+#include "drm_fourcc.h"
 
 // default protected surface tag
 #define PROTECTED_SURFACE_TAG   0x3000f
@@ -111,146 +114,6 @@ void DdiMediaUtil_MediaPrintFps()
 }
 #endif
 
-#ifdef ANDROID
-#define GTT_SIZE_THRESHOLD  (4096*4096*3)    //use the maximum 4K resolution YUV 444 as the threshold
-// The following implemention refers to Solo_PerformManualSwizzling in mos_os_solo.c.
-static __inline int32_t SwizzleOffset(
-    // Linear Offset:
-    int32_t             OffsetX,        // Horizontal byte offset from left edge of tiled surface.
-    int32_t             OffsetY,        // Vertical offset from top of tiled surface.
-    // Swizzling Parameters:
-    int32_t             Pitch,          // Row-to-row byte stride.
-    uint32_t            TileFormat,     // Either 'x' or 'y'--for X-Major or Y-Major tiling, respectively.
-    int32_t             CsxSwizzle)     // (boolean) Additionally perform Channel Select XOR swizzling.
-{
-    int32_t LBits, LPos; // Size and swizzled position of the Line component.
-    if (TileFormat == I915_TILING_NONE)
-    {
-        return(OffsetY * Pitch + OffsetX);
-    }
-
-    if (TileFormat == I915_TILING_Y)
-    {
-        LBits = 5; // Log2(TileY.Height = 32)
-        LPos = 4;  // Log2(TileY.PseudoWidth = 16)
-    }
-    else //if (TileFormat == I915_TILING_X)
-    {
-        LBits = 3; // Log2(TileX.Height = 8)
-        LPos = 9;  // Log2(TileX.Width = 512)
-    }
-
-    int32_t row  =  OffsetY >> LBits;               // OffsetY / LinesPerTile
-    int32_t line =  OffsetY & ((1 << LBits) - 1);   // OffsetY % LinesPerTile
-    int32_t col  =  OffsetX >> LPos;                // OffsetX / BytesPerLine
-    int32_t x    =  OffsetX & ((1 << LPos) - 1);    // OffsetX % BytesPerLine
-
-    int32_t swizzledOffset =
-        (((((row * (Pitch >> LPos)) + col) << LBits) + line) << LPos) + x;
-    //                V                V                 V
-    //                / BytesPerLine   * LinesPerTile    * BytesPerLine
-
-    /// Channel Select XOR Swizzling ///////////////////////////////////////////
-    if (CsxSwizzle)
-    {
-        if (TileFormat == I915_TILING_Y) // A6 = A6 ^ A9
-        {
-            swizzledOffset ^= ((swizzledOffset >> (9 - 6)) & 0x40);
-        }
-        else //if (TileFormat == I915_TILING_X) // A6 = A6 ^ A9 ^ A10
-        {
-            swizzledOffset ^= (((swizzledOffset >> (9 - 6)) ^ (swizzledOffset >> (10 - 6))) & 0x40);
-        }
-    }
-
-    return(swizzledOffset);
-}
-
-static void SwizzleData(
-    uint8_t  *src,
-    uint8_t  *dst,
-    uint32_t  srcTiling,
-    uint32_t  dstTiling,
-    int32_t   height,
-    int32_t   pitch)
-{
-#define IS_TILED(_a)                ((_a) != I915_TILING_NONE)
-#define IS_TILED_TO_LINEAR(_a, _b)  (IS_TILED(_a) && !IS_TILED(_b))
-#define IS_LINEAR_TO_TILED(_a, _b)  (!IS_TILED(_a) && IS_TILED(_b))
-
-    DDI_CHK_NULL(src, "nullptr", );
-    DDI_CHK_NULL(dst, "nullptr", );
-
-    int32_t y,linearOffset,tileOffset;
-    // Translate from one format to another
-    for (y = 0, linearOffset = 0, tileOffset = 0; y < height; y++)
-    {
-        for (int32_t x = 0; x < pitch; x++, linearOffset++)
-        {
-            // x or y --> linear
-            if (IS_TILED_TO_LINEAR(srcTiling, dstTiling))
-            {
-                tileOffset = SwizzleOffset(
-                    x,
-                    y,
-                    pitch,
-                    srcTiling,
-                    false);
-
-                *(dst + linearOffset) = *(src + tileOffset);
-            }
-            // linear --> x or y
-            else if (IS_LINEAR_TO_TILED(srcTiling, dstTiling))
-            {
-                tileOffset = SwizzleOffset(
-                    x,
-                    y,
-                    pitch,
-                    dstTiling,
-                    false);
-
-                *(dst + tileOffset) = *(src + linearOffset);
-            }
-            else
-            {
-                MOS_OS_ASSERT(0);
-            }
-        }
-    }
-}
-
-static bool NeedSwizzleData(PDDI_MEDIA_SURFACE surface, bool lock)
-{
-    DDI_CHK_NULL(surface, "nullptr surface", false);
-    DDI_CHK_NULL(surface->pGmmResourceInfo, "nullptr pGmmResourceInfo", false);
-    uint32_t pitch = (uint32_t)surface->pGmmResourceInfo->GetRenderPitch();
-    uint32_t size  = (uint32_t)surface->pGmmResourceInfo->GetSizeSurface();
-    GMM_RESOURCE_FLAG gmmFlags = surface->pGmmResourceInfo->GetResFlags();
-
-    if (gmmFlags.Gpu.RenderTarget      &&
-        gmmFlags.Gpu.UnifiedAuxSurface &&
-        gmmFlags.Gpu.CCS)
-    {
-        size = size - (uint32_t)(surface->pGmmResourceInfo->GetSizeAuxSurface(GMM_AUX_SURF));
-    }
-
-    uint8_t *resourceBase = (uint8_t*)MOS_AllocAndZeroMemory(size);
-    DDI_CHK_NULL(resourceBase, "nullptr resourceBase", false);
-
-    if (lock)
-    {
-        SwizzleData((uint8_t*) surface->bo->virt, resourceBase, surface->TileType, I915_TILING_NONE, size / pitch, pitch);
-    }
-    else
-    {
-        SwizzleData((uint8_t*) surface->bo->virt, resourceBase, I915_TILING_NONE, surface->TileType, size / pitch, pitch);
-    }
-    MOS_SecureMemcpy((uint8_t*) surface->bo->virt, size, resourceBase, size);
-    MOS_FreeMemory(resourceBase);
-
-    return true;
-}
-#endif
 /*
  * DdiMediaUtil_IsExternalSurface
  *    Descripion: if the bo of media surface was allocated from App,
@@ -299,7 +162,6 @@ VAStatus DdiMediaUtil_AllocateSurface(
     MOS_LINUX_BO               *bo = nullptr;
     GMM_RESCREATE_PARAMS        gmmParams;
     GMM_RESOURCE_INFO          *gmmResourceInfo = nullptr;
-    bool                        grallocAllocation;
 
     DDI_CHK_NULL(mediaSurface, "mediaSurface is nullptr", VA_STATUS_ERROR_INVALID_BUFFER);
     DDI_CHK_NULL(mediaDrvCtx, "mediaDrvCtx is nullptr", VA_STATUS_ERROR_INVALID_BUFFER);
@@ -308,8 +170,12 @@ VAStatus DdiMediaUtil_AllocateSurface(
     int32_t size          = 0;
     uint32_t tileformat   = I915_TILING_NONE;
     VAStatus hRes         = VA_STATUS_SUCCESS;
+    int32_t alignedWidth  = width;
     int32_t alignedHeight = height;
     uint32_t tag          = 0;
+    int mem_type          = MOS_MEMPOOL_VIDEOMEMORY;
+    bool bMemCompEnable   = true;
+    bool bMemCompRC       = false;
 
     switch (format)
     {
@@ -321,30 +187,37 @@ VAStatus DdiMediaUtil_AllocateSurface(
         case Media_Format_R8G8B8:
         case Media_Format_R10G10B10A2:
         case Media_Format_B10G10R10A2:
-            if (VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER != mediaSurface->surfaceUsageHint)
+        case Media_Format_R10G10B10X2:
+        case Media_Format_B10G10R10X2:
+        case Media_Format_A16R16G16B16:
+        case Media_Format_A16B16G16R16:
+            if (VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER != mediaSurface->surfaceUsageHint   &&
+                !(mediaSurface->surfaceUsageHint & VA_SURFACE_ATTRIB_USAGE_HINT_VPP_WRITE))
             {
-                 tileformat = I915_TILING_NONE;
-                 break;
+                tileformat = I915_TILING_NONE;
+                break;
             }
-        case Media_Format_NV21:
         case Media_Format_YV12:
         case Media_Format_I420:
         case Media_Format_IYUV:
-            if (VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER != mediaSurface->surfaceUsageHint)
+            if (VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER != mediaSurface->surfaceUsageHint   &&
+                !(mediaSurface->surfaceUsageHint & VA_SURFACE_ATTRIB_USAGE_HINT_VPP_WRITE))
             {
-                 tileformat = I915_TILING_NONE;
-                 break;
+                tileformat = I915_TILING_NONE;
+                break;
             }
         case Media_Format_RGBP:
-        case Media_Format_UYVY:
+        case Media_Format_BGRP:
         case Media_Format_A8R8G8B8:
             if (VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER != mediaSurface->surfaceUsageHint &&
-                !(mediaSurface->surfaceUsageHint & VA_SURFACE_ATTRIB_USAGE_HINT_DECODER))
+                !(mediaSurface->surfaceUsageHint & VA_SURFACE_ATTRIB_USAGE_HINT_DECODER) &&
+                !(mediaSurface->surfaceUsageHint & VA_SURFACE_ATTRIB_USAGE_HINT_VPP_WRITE))
             {
-                 tileformat = I915_TILING_NONE;
-                 break;
+                tileformat = I915_TILING_NONE;
+                break;
             }
         case Media_Format_NV12:
+        case Media_Format_NV21:
         case Media_Format_444P:
         case Media_Format_422H:
         case Media_Format_411P:
@@ -352,6 +225,7 @@ VAStatus DdiMediaUtil_AllocateSurface(
         case Media_Format_IMC3:
         case Media_Format_400P:
         case Media_Format_P010:
+        case Media_Format_P012:
         case Media_Format_P016:
         case Media_Format_YUY2:
         case Media_Format_Y210:
@@ -359,17 +233,47 @@ VAStatus DdiMediaUtil_AllocateSurface(
         case Media_Format_AYUV:
         case Media_Format_Y410:
         case Media_Format_Y416:
-            if (VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER != mediaSurface->surfaceUsageHint)
+        case Media_Format_Y8:
+        case Media_Format_Y16S:
+        case Media_Format_Y16U:
+        case Media_Format_VYUY:
+        case Media_Format_YVYU:
+        case Media_Format_UYVY:
+            if (VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER != mediaSurface->surfaceUsageHint &&
+                !(mediaSurface->surfaceUsageHint & VA_SURFACE_ATTRIB_USAGE_HINT_VPP_WRITE))
             {
 #if UFO_GRALLOC_NEW_FORMAT
-                 //Planar type surface align 64 to improve performance.
+                //Planar type surface align 64 to improve performance.
                 alignedHeight = MOS_ALIGN_CEIL(height, 64);
 #else
                 //Planar type surface align 32 to improve performance.
                 alignedHeight = MOS_ALIGN_CEIL(height, 32);
 #endif
             }
-            tileformat  = I915_TILING_Y;
+            alignedWidth = MOS_ALIGN_CEIL(width, 8);
+            if (mediaSurface->surfaceUsageHint & VA_SURFACE_ATTRIB_USAGE_HINT_VPP_WRITE)
+            {
+                if ((format == Media_Format_YV12) ||
+                    (format == Media_Format_I420))
+                {
+                    alignedWidth = MOS_ALIGN_CEIL(width, 128);
+                }
+
+                if ((format == Media_Format_NV12) ||
+                    (format == Media_Format_P010) ||
+                    (format == Media_Format_RGBP) ||
+                    (format == Media_Format_BGRP))
+                {
+#if UFO_GRALLOC_NEW_FORMAT
+                    //Planar type surface align 64 to improve performance.
+                    alignedHeight = MOS_ALIGN_CEIL(height, 64);
+#else
+                    //Planar type surface align 32 to improve performance.
+                    alignedHeight = MOS_ALIGN_CEIL(height, 32);
+#endif
+                }
+            }
+            tileformat = I915_TILING_Y;
             break;
         case Media_Format_Buffer:
             tileformat = I915_TILING_NONE;
@@ -387,11 +291,10 @@ VAStatus DdiMediaUtil_AllocateSurface(
             tag = PROTECTED_SURFACE_TAG;
         }
         // DRM buffer allocated by Application, No need to re-allocate new DRM buffer
-         if( (mediaSurface->pSurfDesc->uiFlags & VA_SURFACE_ATTRIB_MEM_TYPE_KERNEL_DRM)
-             || (mediaSurface->pSurfDesc->uiFlags & VA_SURFACE_ATTRIB_MEM_TYPE_DRM_PRIME)
-           )
+        if((mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_KERNEL_DRM) ||
+           (mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_DRM_PRIME))
         {
-            if (mediaSurface->pSurfDesc->uiFlags & VA_SURFACE_ATTRIB_MEM_TYPE_KERNEL_DRM)
+            if (mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_KERNEL_DRM)
             {
                 bo = mos_bo_gem_create_from_name(mediaDrvCtx->pDrmBufMgr, "MEDIA", mediaSurface->pSurfDesc->ulBuffer);
             }
@@ -415,7 +318,50 @@ VAStatus DdiMediaUtil_AllocateSurface(
                 return VA_STATUS_ERROR_ALLOCATION_FAILED;
             }
         }
-        else if( mediaSurface->pSurfDesc->uiFlags & VA_SURFACE_ATTRIB_MEM_TYPE_USER_PTR )
+        else if (mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_DRM_PRIME_2)
+        {
+            bo = mos_bo_gem_create_from_prime(mediaDrvCtx->pDrmBufMgr, mediaSurface->pSurfDesc->ulBuffer, mediaSurface->pSurfDesc->uiSize);
+            if( bo != nullptr )
+            {
+                pitch = mediaSurface->pSurfDesc->uiPitches[0];
+                switch (mediaSurface->pSurfDesc->modifier)
+                {
+                    case DRM_FORMAT_MOD_LINEAR:
+                        tileformat = I915_TILING_NONE;
+                        bMemCompEnable = false;
+                        break;
+                    case I915_FORMAT_MOD_X_TILED:
+                        tileformat = I915_TILING_X;
+                        bMemCompEnable = false;
+                        break;
+                    case I915_FORMAT_MOD_Y_TILED:
+                    case I915_FORMAT_MOD_Yf_TILED:
+                        tileformat = I915_TILING_Y;
+                        bMemCompEnable = false;
+                        break;
+                    case I915_FORMAT_MOD_Y_TILED_GEN12_RC_CCS:
+                        tileformat = I915_TILING_Y;
+                        bMemCompEnable = true;
+                        bMemCompRC = true;
+                        break;
+                    case I915_FORMAT_MOD_Y_TILED_GEN12_MC_CCS:
+                        tileformat = I915_TILING_Y;
+                        bMemCompEnable = true;
+                        bMemCompRC = false;
+                        break;
+                    default:
+                        DDI_ASSERTMESSAGE("Unsupported modifier.");
+                        hRes = VA_STATUS_ERROR_INVALID_PARAMETER;
+                        goto finish;
+                }
+            }
+            else
+            {
+                DDI_ASSERTMESSAGE("Failed to create drm buffer object according to input buffer descriptor.");
+                return VA_STATUS_ERROR_ALLOCATION_FAILED;
+            }
+        }
+        else if( mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_USER_PTR )
         {
 
             pitch    = mediaSurface->pSurfDesc->uiPitches[0];
@@ -453,8 +399,15 @@ VAStatus DdiMediaUtil_AllocateSurface(
                return VA_STATUS_ERROR_ALLOCATION_FAILED;
            }
         }
-        else
+        else if( mediaSurface->pSurfDesc->uiFlags & VA_SURFACE_EXTBUF_DESC_ENABLE_TILING )
         {
+            tileformat = I915_TILING_Y;
+        }
+        else if (mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_VA)
+        {
+            tileformat = I915_TILING_NONE;
+            alignedHeight = height;
+        } else {
             DDI_ASSERTMESSAGE("Input buffer descriptor (%d) is not supported by current driver.", mediaSurface->pSurfDesc->uiFlags);
             return VA_STATUS_ERROR_ALLOCATION_FAILED;
         }
@@ -462,20 +415,37 @@ VAStatus DdiMediaUtil_AllocateSurface(
 
     // Create GmmResourceInfo
     MOS_ZeroMemory(&gmmParams, sizeof(gmmParams));
-    if (DdiMediaUtil_IsExternalSurface(mediaSurface))
+    if (DdiMediaUtil_IsExternalSurface(mediaSurface) &&
+        mediaSurface->pSurfDesc->uiVaMemType != VA_SURFACE_ATTRIB_MEM_TYPE_VA)
     {
         gmmParams.BaseWidth         = mediaSurface->iWidth;
         gmmParams.BaseHeight        = mediaSurface->iHeight;
+        if(mediaSurface->pSurfDesc->uiPlanes > 0 && mediaSurface->pSurfDesc->uiPitches[0] > 0)
+        {
+            if((mediaSurface->pSurfDesc->uiPlanes > 1) && (mediaSurface->pSurfDesc->uiOffsets[1] > mediaSurface->pSurfDesc->uiOffsets[0]))
+            {
+                gmmParams.BaseHeight = (mediaSurface->pSurfDesc->uiOffsets[1] - mediaSurface->pSurfDesc->uiOffsets[0]) / mediaSurface->pSurfDesc->uiPitches[0];
+            }
+            else if(mediaSurface->pSurfDesc->uiSize > 0)
+            {
+                gmmParams.BaseHeight = mediaSurface->pSurfDesc->uiSize / mediaSurface->pSurfDesc->uiPitches[0];
+            }
+        }
+        if(mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_USER_PTR)
+        {
+            gmmParams.ExistingSysMemSize = mediaSurface->pSurfDesc->uiBuffserSize;
+            gmmParams.pExistingSysMem    = mediaSurface->pSurfDesc->ulBuffer;
+            gmmParams.Flags.Info.ExistingSysMem = true;
+        }
     }
     else
     {
-        gmmParams.BaseWidth         = width;
+        gmmParams.BaseWidth         = alignedWidth;
         gmmParams.BaseHeight        = alignedHeight;
     }
 
     gmmParams.ArraySize             = 1;
     gmmParams.Type                  = RESOURCE_2D;
-    //gmmParams.Format                = DdiMediaUtil_ConvertMediaFmtToGmmFmt(format);
     gmmParams.Format                = mediaDrvCtx->m_caps->ConvertMediaFmtToGmmFmt(format);
     gmmParams.CpTag                 = tag;
 
@@ -487,13 +457,28 @@ VAStatus DdiMediaUtil_AllocateSurface(
         case I915_TILING_Y:
             // Disable MMC for application required surfaces, because some cases' output streams have corruption.
             gmmParams.Flags.Gpu.MMC    = false;
-            if ( mediaDrvCtx->m_auxTableMgr )
+            if (MEDIA_IS_SKU(&mediaDrvCtx->SkuTable, FtrE2ECompression) &&
+                (!MEDIA_IS_WA(&mediaDrvCtx->WaTable, WaDisableVPMmc)    &&
+                !MEDIA_IS_WA(&mediaDrvCtx->WaTable, WaDisableCodecMmc)) &&
+                bMemCompEnable)
             {
-                gmmParams.Flags.Gpu.MMC = true;
-                gmmParams.Flags.Info.MediaCompressed = 1;
-                gmmParams.Flags.Gpu.CCS = 1;
+                gmmParams.Flags.Gpu.MMC               = true;
+                gmmParams.Flags.Info.MediaCompressed  = 1;
+                gmmParams.Flags.Info.RenderCompressed = 0;
+                gmmParams.Flags.Gpu.CCS               = 1;
+                gmmParams.Flags.Gpu.RenderTarget      = 1;
                 gmmParams.Flags.Gpu.UnifiedAuxSurface = 1;
-                gmmParams.Flags.Gpu.RenderTarget = 1;
+
+                if (bMemCompRC)
+                {
+                    gmmParams.Flags.Info.MediaCompressed  = 0;
+                    gmmParams.Flags.Info.RenderCompressed = 1;
+                }
+
+                if(MEDIA_IS_SKU(&mediaDrvCtx->SkuTable, FtrFlatPhysCCS))
+                {
+                    gmmParams.Flags.Gpu.UnifiedAuxSurface = 0;
+                }
             }
             break;
         case I915_TILING_X:
@@ -517,6 +502,12 @@ VAStatus DdiMediaUtil_AllocateSurface(
     uint32_t    gmmPitch;
     uint32_t    gmmSize;
     uint32_t    gmmHeight;
+
+    gmmPitch    = (uint32_t)gmmResourceInfo->GetRenderPitch();
+    if( DdiMediaUtil_IsExternalSurface(mediaSurface) && mediaSurface->pSurfDesc->uiPitches[0])
+    {
+        gmmResourceInfo->OverridePitch(mediaSurface->pSurfDesc->uiPitches[0]);
+    }
     gmmPitch    = (uint32_t)gmmResourceInfo->GetRenderPitch();
     gmmSize     = (uint32_t)gmmResourceInfo->GetSizeSurface();
     gmmHeight   = gmmResourceInfo->GetBaseHeight();
@@ -528,32 +519,47 @@ VAStatus DdiMediaUtil_AllocateSurface(
         goto finish;
     }
 
-    if (!DdiMediaUtil_IsExternalSurface(mediaSurface))
+    mem_type = MemoryPolicyManager::UpdateMemoryPolicy(&mediaDrvCtx->SkuTable, mediaSurface->pGmmResourceInfo);
+
+    if (!DdiMediaUtil_IsExternalSurface(mediaSurface) ||
+        mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_VA)
     {
+        switch (gmmResourceInfo->GetTileType())
+        {
+            case GMM_TILED_Y:
+                tileformat = I915_TILING_Y;
+                break;
+            case GMM_TILED_X:
+                tileformat = I915_TILING_X;
+                break;
+            case GMM_NOT_TILED:
+                tileformat = I915_TILING_NONE;
+                break;
+            default:
+                tileformat = I915_TILING_Y;
+                break;
+        }
+
         unsigned long  ulPitch = 0;
         if ( tileformat == I915_TILING_NONE )
         {
-            bo = mos_bo_alloc(mediaDrvCtx->pDrmBufMgr, "MEDIA", gmmSize, 4096);
+            bo = mos_bo_alloc(mediaDrvCtx->pDrmBufMgr, "MEDIA", gmmSize, 4096, mem_type);
             pitch = gmmPitch;
         }
         else
         {
-            bo = mos_bo_alloc_tiled(mediaDrvCtx->pDrmBufMgr, "MEDIA", gmmPitch, gmmSize/gmmPitch, 1, &tileformat, (unsigned long *)&ulPitch, 0);
+            bo = mos_bo_alloc_tiled(mediaDrvCtx->pDrmBufMgr, "MEDIA", gmmPitch, (gmmSize + gmmPitch -1)/gmmPitch, 1, &tileformat, (unsigned long *)&ulPitch, 0, mem_type);
             pitch = ulPitch;
         }
     }
-    else if(mediaSurface->pSurfDesc->uiFlags & VA_SURFACE_ATTRIB_MEM_TYPE_USER_PTR)
+    else if(mediaSurface->pSurfDesc->uiVaMemType == VA_SURFACE_ATTRIB_MEM_TYPE_USER_PTR)
     {
         gmmHeight = height;
     }
     else
     {
-        // Check Pitch and Size
-#ifdef ANDROID
+        // Check Pitch
         if (gmmPitch > pitch)
-#else
-        if (gmmPitch > pitch || gmmSize > bo->size)
-#endif
         {
             DDI_ASSERTMESSAGE("External Surface doesn't meet the reqirements of Media driver.");
             DdiMediaUtil_FreeSurface(mediaSurface);
@@ -632,25 +638,7 @@ VAStatus DdiMediaUtil_AllocateBuffer(
        return VA_STATUS_ERROR_INVALID_PARAMETER;
 
     VAStatus     hRes = VA_STATUS_SUCCESS;
-    MOS_LINUX_BO *bo  = mos_bo_alloc(bufmgr, "Media Buffer", size, 4096);
-
-    mediaBuffer->bMapped = false;
-    if (bo)
-    {
-        mediaBuffer->format     = format;
-        mediaBuffer->iSize      = size;
-        mediaBuffer->iRefCount  = 0;
-        mediaBuffer->bo         = bo;
-        mediaBuffer->pData      = (uint8_t*) bo->virt;
-
-        DDI_VERBOSEMESSAGE("Alloc %7d bytes resource.",size);
-    }
-    else
-    {
-        DDI_ASSERTMESSAGE("Fail to Alloc %7d bytes resource.",size);
-        hRes = VA_STATUS_ERROR_ALLOCATION_FAILED;
-        goto finish;
-    }
+    int32_t          mem_type = MOS_MEMPOOL_VIDEOMEMORY;
 
     // create fake GmmResourceInfo
     GMM_RESCREATE_PARAMS    gmmParams;
@@ -671,6 +659,28 @@ VAStatus DdiMediaUtil_AllocateBuffer(
     mediaBuffer->pGmmResourceInfo->OverrideSize(mediaBuffer->iSize);
     mediaBuffer->pGmmResourceInfo->OverrideBaseWidth(mediaBuffer->iSize);
     mediaBuffer->pGmmResourceInfo->OverridePitch(mediaBuffer->iSize);
+
+    mem_type = MemoryPolicyManager::UpdateMemoryPolicy(&mediaBuffer->pMediaCtx->SkuTable, mediaBuffer->pGmmResourceInfo);
+    MOS_LINUX_BO *bo  = mos_bo_alloc(bufmgr, "Media Buffer", size, 4096, mem_type);
+
+    mediaBuffer->bMapped = false;
+    if (bo)
+    {
+        mediaBuffer->format     = format;
+        mediaBuffer->iSize      = size;
+        mediaBuffer->iRefCount  = 0;
+        mediaBuffer->bo         = bo;
+        mediaBuffer->pData      = (uint8_t*) bo->virt;
+
+        DDI_VERBOSEMESSAGE("Alloc %8d bytes resource.",size);
+    }
+    else
+    {
+        DDI_ASSERTMESSAGE("Fail to Alloc %8d bytes resource.",size);
+        hRes = VA_STATUS_ERROR_ALLOCATION_FAILED;
+        goto finish;
+    }
+
 finish:
     return hRes;
 }
@@ -703,6 +713,7 @@ VAStatus DdiMediaUtil_Allocate2DBuffer(
     int32_t  size           = 0;
     uint32_t tileformat     = I915_TILING_NONE;
     VAStatus hRes           = VA_STATUS_SUCCESS;
+    int32_t  mem_type       = MOS_MEMPOOL_VIDEOMEMORY;
 
     // Create GmmResourceInfo
     GMM_RESCREATE_PARAMS        gmmParams;
@@ -737,8 +748,10 @@ VAStatus DdiMediaUtil_Allocate2DBuffer(
     gmmSize     = (uint32_t)gmmResourceInfo->GetSizeSurface();
     gmmHeight   = gmmResourceInfo->GetBaseHeight();
 
+    mem_type = MemoryPolicyManager::UpdateMemoryPolicy(&mediaBuffer->pMediaCtx->SkuTable, mediaBuffer->pGmmResourceInfo);
+
     MOS_LINUX_BO  *bo;
-    bo = mos_bo_alloc(bufmgr, "Media 2D Buffer", gmmSize, 4096);
+    bo = mos_bo_alloc(bufmgr, "Media 2D Buffer", gmmSize, 4096, mem_type);
 
     mediaBuffer->bMapped = false;
     if (bo)
@@ -822,6 +835,8 @@ VAStatus DdiMediaUtil_CreateBuffer(DDI_MEDIA_BUFFER *buffer, MOS_BUFMGR *bufmgr)
     return hr;
 }
 
+VAStatus SwizzleSurface(PDDI_MEDIA_CONTEXT mediaCtx, PGMM_RESOURCE_INFO pGmmResInfo, void *pLockedAddr, uint32_t TileType, uint8_t* pResourceBase, bool bUpload);
+
 // add thread protection for multiple thread?
 void* DdiMediaUtil_LockSurface(DDI_MEDIA_SURFACE  *surface, uint32_t flag)
 {
@@ -831,23 +846,7 @@ void* DdiMediaUtil_LockSurface(DDI_MEDIA_SURFACE  *surface, uint32_t flag)
     {
         if (surface->pMediaCtx->bIsAtomSOC)
         {
-#ifdef ANDROID
-            if (surface->iWidth * surface->iHeight * 3 <= GTT_SIZE_THRESHOLD)
-            {
-                mos_gem_bo_map_gtt(surface->bo);
-            }
-            else
-            {
-                mos_bo_map(surface->bo, (MOS_LOCKFLAG_READONLY | MOS_LOCKFLAG_WRITEONLY));
-                if (surface->TileType != I915_TILING_NONE)
-                {
-                    if (NeedSwizzleData(surface, true) == false)
-                        return nullptr;
-                }
-            }
-#else
             mos_gem_bo_map_gtt(surface->bo);
-#endif
         }
         else
         {
@@ -855,7 +854,7 @@ void* DdiMediaUtil_LockSurface(DDI_MEDIA_SURFACE  *surface, uint32_t flag)
             {
                 mos_bo_map(surface->bo, flag & MOS_LOCKFLAG_WRITEONLY);
             }
-            else if (surface->pMediaCtx->m_useSwSwizzling)
+            else if ((surface->pMediaCtx->m_useSwSwizzling) && !(flag & MOS_LOCKFLAG_NO_SWIZZLE))
             {
                 mos_bo_map(surface->bo, flag & MOS_LOCKFLAG_WRITEONLY);
                 if (surface->pSystemShadow == nullptr)
@@ -863,26 +862,28 @@ void* DdiMediaUtil_LockSurface(DDI_MEDIA_SURFACE  *surface, uint32_t flag)
                     surface->pSystemShadow = (uint8_t*)MOS_AllocMemory(surface->bo->size);
                     DDI_CHK_CONDITION((surface->pSystemShadow == nullptr), "Failed to allocate shadow surface", nullptr);
                 }
-                if (surface->pSystemShadow)
-                {
-                    int32_t flags = surface->pMediaCtx->m_tileYFlag ? 0 : 1;
-                    uint64_t surfSize = surface->pGmmResourceInfo->GetSizeMainSurface();
-                    DDI_CHK_CONDITION((surface->TileType != I915_TILING_Y), "Unsupported tile type", nullptr);
-                    DDI_CHK_CONDITION((surfSize <= 0 || surface->iPitch <= 0), "Invalid surface size or pitch", nullptr);
 
-                    Mos_SwizzleData((uint8_t*)surface->bo->virt, (uint8_t*)surface->pSystemShadow,
-                                    MOS_TILE_Y, MOS_TILE_LINEAR,
-                                    static_cast<int32_t>(surfSize / surface->iPitch), surface->iPitch, flags);
-                }
-            }
-            else if (flag & MOS_LOCKFLAG_WRITEONLY)
-            {
-                mos_gem_bo_map_gtt(surface->bo);
+                uint64_t surfSize = surface->pGmmResourceInfo->GetSizeMainSurface();
+                DDI_CHK_CONDITION((surface->TileType != I915_TILING_Y), "Unsupported tile type", nullptr);
+                DDI_CHK_CONDITION((surfSize <= 0 || surface->iPitch <= 0), "Invalid surface size or pitch", nullptr);
+
+                VAStatus vaStatus = SwizzleSurface(surface->pMediaCtx,
+                                                   surface->pGmmResourceInfo,
+                                                   surface->bo->virt,
+                                                   (MOS_TILE_TYPE)surface->TileType,
+                                                   (uint8_t *)surface->pSystemShadow,
+                                                   false);
+                DDI_CHK_CONDITION((vaStatus != VA_STATUS_SUCCESS), "SwizzleSurface failed", nullptr);
+
             }
             else if (flag & MOS_LOCKFLAG_NO_SWIZZLE)
             {
                 mos_bo_map(surface->bo, flag & MOS_LOCKFLAG_READONLY);
             }
+            else if (flag & MOS_LOCKFLAG_WRITEONLY)
+            {
+                mos_gem_bo_map_gtt(surface->bo);
+            }
             else
             {
                 mos_gem_bo_map_unsynchronized(surface->bo);     // only call mmap_gtt ioctl
@@ -914,21 +915,7 @@ void DdiMediaUtil_UnlockSurface(DDI_MEDIA_SURFACE  *surface)
     {
         if (surface->pMediaCtx->bIsAtomSOC)
         {
-#ifdef ANDROID
-            if (surface->iWidth * surface->iHeight * 3 <= GTT_SIZE_THRESHOLD)
-            {
-                mos_gem_bo_unmap_gtt(surface->bo);
-            }
-            else
-            {
-                if (surface->TileType != I915_TILING_NONE)
-                    NeedSwizzleData(surface, false);
-
-                mos_bo_unmap(surface->bo);
-            }
-#else
             mos_gem_bo_unmap_gtt(surface->bo);
-#endif
         }
         else
         {
@@ -938,12 +925,13 @@ void DdiMediaUtil_UnlockSurface(DDI_MEDIA_SURFACE  *surface)
             }
             else if (surface->pSystemShadow)
             {
-                int32_t flags = surface->pMediaCtx->m_tileYFlag ? 0 : 1;
-                uint64_t surfSize = surface->pGmmResourceInfo->GetSizeMainSurface();
+                SwizzleSurface(surface->pMediaCtx,
+                               surface->pGmmResourceInfo,
+                               surface->bo->virt,
+                               (MOS_TILE_TYPE)surface->TileType,
+                               (uint8_t *)surface->pSystemShadow,
+                               true);
 
-                Mos_SwizzleData((uint8_t*)surface->pSystemShadow, (uint8_t*)surface->bo->virt,
-                                MOS_TILE_LINEAR, MOS_TILE_Y,
-                                static_cast<int32_t>(surfSize / surface->iPitch), surface->iPitch, flags);
                 MOS_FreeMemory(surface->pSystemShadow);
                 surface->pSystemShadow = nullptr;
 
@@ -1115,8 +1103,6 @@ void DdiMediaUtil_FreeSurface(DDI_MEDIA_SURFACE *surface)
 void DdiMediaUtil_FreeBuffer(DDI_MEDIA_BUFFER  *buf)
 {
     DDI_CHK_NULL(buf, "nullptr", );
-    DDI_CHK_NULL(buf->pMediaCtx, "nullptr", );
-    DDI_CHK_NULL(buf->pMediaCtx->pGmmClientContext, "nullptr", );
     // calling sequence checking
     if (buf->bMapped)
     {
@@ -1134,7 +1120,7 @@ void DdiMediaUtil_FreeBuffer(DDI_MEDIA_BUFFER  *buf)
         buf->bo = nullptr;
     }
 
-    if (nullptr != buf->pGmmResourceInfo)
+    if (nullptr != buf->pMediaCtx && nullptr != buf->pMediaCtx->pGmmClientContext && nullptr != buf->pGmmResourceInfo)
     {
         buf->pMediaCtx->pGmmClientContext->DestroyResInfoObject(buf->pGmmResourceInfo);
         buf->pGmmResourceInfo = nullptr;
